"""
  Generated by Eclipse Cyclone DDS idlc Python Backend
  Cyclone DDS IDL version: v0.11.0
  Module: DDS.XTypes
  IDL file: ddsi_xt_type_object.idl

"""

from dataclasses import dataclass
from enum import auto
from typing import TYPE_CHECKING, Optional

import cyclonedds.idl as idl
import cyclonedds.idl.annotations as annotate
import cyclonedds.idl.types as types

# root module import for resolving types
import cyclonedds.idl._typesupport.DDS

EquivalenceKind = types.typedef['cyclonedds.idl._typesupport.DDS.XTypes.EquivalenceKind', types.byte]
EK_MINIMAL = 241
EK_COMPLETE = 242
EK_BOTH = 243
TypeKind = types.typedef['cyclonedds.idl._typesupport.DDS.XTypes.TypeKind', types.byte]
TK_NONE = 0
TK_BOOLEAN = 1
TK_BYTE = 2
TK_INT16 = 3
TK_INT32 = 4
TK_INT64 = 5
TK_UINT16 = 6
TK_UINT32 = 7
TK_UINT64 = 8
TK_FLOAT32 = 9
TK_FLOAT64 = 10
TK_FLOAT128 = 11
TK_INT8 = 12
TK_UINT8 = 13
TK_CHAR8 = 16
TK_CHAR16 = 17
TK_STRING8 = 32
TK_STRING16 = 33
TK_ALIAS = 48
TK_ENUM = 64
TK_BITMASK = 65
TK_ANNOTATION = 80
TK_STRUCTURE = 81
TK_UNION = 82
TK_BITSET = 83
TK_SEQUENCE = 96
TK_ARRAY = 97
TK_MAP = 98
TypeIdentiferKind = types.typedef['cyclonedds.idl._typesupport.DDS.XTypes.TypeIdentiferKind', types.byte]
TI_STRING8_SMALL = 112
TI_STRING8_LARGE = 113
TI_STRING16_SMALL = 114
TI_STRING16_LARGE = 115
TI_PLAIN_SEQUENCE_SMALL = 128
TI_PLAIN_SEQUENCE_LARGE = 129
TI_PLAIN_ARRAY_SMALL = 144
TI_PLAIN_ARRAY_LARGE = 145
TI_PLAIN_MAP_SMALL = 160
TI_PLAIN_MAP_LARGE = 161
TI_STRONGLY_CONNECTED_COMPONENT = 176
MEMBER_NAME_MAX_LENGTH = 256
MemberName = types.typedef['cyclonedds.idl._typesupport.DDS.XTypes.MemberName', types.bounded_str[256]]
TYPE_NAME_MAX_LENGTH = 256
QualifiedTypeName = types.typedef['cyclonedds.idl._typesupport.DDS.XTypes.QualifiedTypeName', types.bounded_str[256]]
PrimitiveTypeId = types.typedef['cyclonedds.idl._typesupport.DDS.XTypes.PrimitiveTypeId', types.byte]
EquivalenceHash = types.typedef['cyclonedds.idl._typesupport.DDS.XTypes.EquivalenceHash', types.array[types.byte, 14]]
NameHash = types.typedef['cyclonedds.idl._typesupport.DDS.XTypes.NameHash', types.array[types.byte, 4]]
LBound = types.typedef['cyclonedds.idl._typesupport.DDS.XTypes.LBound', types.uint32]
LBoundSeq = types.typedef['cyclonedds.idl._typesupport.DDS.XTypes.LBoundSeq', types.sequence['cyclonedds.idl._typesupport.DDS.XTypes.LBound']]
INVALID_LBOUND = 0
SBound = types.typedef['cyclonedds.idl._typesupport.DDS.XTypes.SBound', types.byte]
SBoundSeq = types.typedef['cyclonedds.idl._typesupport.DDS.XTypes.SBoundSeq', types.sequence['cyclonedds.idl._typesupport.DDS.XTypes.SBound']]
INVALID_SBOUND = 0


@annotate.final
@annotate.nested
class TypeObjectHashId(idl.IdlUnion, discriminator=types.byte, discriminator_is_key=False, typename="DDS.XTypes.TypeObjectHashId"):
    hash: types.case[[242, 241], 'cyclonedds.idl._typesupport.DDS.XTypes.EquivalenceHash']


@dataclass
@annotate.bit_bound(16)
class MemberFlag(idl.IdlBitmask, typename="DDS.XTypes.MemberFlag"):
    TRY_CONSTRUCT1: bool = False
    annotate.position("TRY_CONSTRUCT1", 0)
    TRY_CONSTRUCT2: bool = False
    annotate.position("TRY_CONSTRUCT2", 1)
    IS_EXTERNAL: bool = False
    annotate.position("IS_EXTERNAL", 2)
    IS_OPTIONAL: bool = False
    annotate.position("IS_OPTIONAL", 3)
    IS_MUST_UNDERSTAND: bool = False
    annotate.position("IS_MUST_UNDERSTAND", 4)
    IS_KEY: bool = False
    annotate.position("IS_KEY", 5)
    IS_DEFAULT: bool = False
    annotate.position("IS_DEFAULT", 6)

CollectionElementFlag = types.typedef['cyclonedds.idl._typesupport.DDS.XTypes.CollectionElementFlag', 'cyclonedds.idl._typesupport.DDS.XTypes.MemberFlag']
StructMemberFlag = types.typedef['cyclonedds.idl._typesupport.DDS.XTypes.StructMemberFlag', 'cyclonedds.idl._typesupport.DDS.XTypes.MemberFlag']
UnionMemberFlag = types.typedef['cyclonedds.idl._typesupport.DDS.XTypes.UnionMemberFlag', 'cyclonedds.idl._typesupport.DDS.XTypes.MemberFlag']
UnionDiscriminatorFlag = types.typedef['cyclonedds.idl._typesupport.DDS.XTypes.UnionDiscriminatorFlag', 'cyclonedds.idl._typesupport.DDS.XTypes.MemberFlag']
EnumeratedLiteralFlag = types.typedef['cyclonedds.idl._typesupport.DDS.XTypes.EnumeratedLiteralFlag', 'cyclonedds.idl._typesupport.DDS.XTypes.MemberFlag']
AnnotationParameterFlag = types.typedef['cyclonedds.idl._typesupport.DDS.XTypes.AnnotationParameterFlag', 'cyclonedds.idl._typesupport.DDS.XTypes.MemberFlag']
AliasMemberFlag = types.typedef['cyclonedds.idl._typesupport.DDS.XTypes.AliasMemberFlag', 'cyclonedds.idl._typesupport.DDS.XTypes.MemberFlag']
BitflagFlag = types.typedef['cyclonedds.idl._typesupport.DDS.XTypes.BitflagFlag', 'cyclonedds.idl._typesupport.DDS.XTypes.MemberFlag']
BitsetMemberFlag = types.typedef['cyclonedds.idl._typesupport.DDS.XTypes.BitsetMemberFlag', 'cyclonedds.idl._typesupport.DDS.XTypes.MemberFlag']
MemberFlagMinimalMask = 63

@dataclass
@annotate.bit_bound(16)
class TypeFlag(idl.IdlBitmask, typename="DDS.XTypes.TypeFlag"):
    IS_FINAL: bool = False
    annotate.position("IS_FINAL", 0)
    IS_APPENDABLE: bool = False
    annotate.position("IS_APPENDABLE", 1)
    IS_MUTABLE: bool = False
    annotate.position("IS_MUTABLE", 2)
    IS_NESTED: bool = False
    annotate.position("IS_NESTED", 3)
    IS_AUTOID_HASH: bool = False
    annotate.position("IS_AUTOID_HASH", 4)

StructTypeFlag = types.typedef['cyclonedds.idl._typesupport.DDS.XTypes.StructTypeFlag', 'cyclonedds.idl._typesupport.DDS.XTypes.TypeFlag']
UnionTypeFlag = types.typedef['cyclonedds.idl._typesupport.DDS.XTypes.UnionTypeFlag', 'cyclonedds.idl._typesupport.DDS.XTypes.TypeFlag']
CollectionTypeFlag = types.typedef['cyclonedds.idl._typesupport.DDS.XTypes.CollectionTypeFlag', 'cyclonedds.idl._typesupport.DDS.XTypes.TypeFlag']
AnnotationTypeFlag = types.typedef['cyclonedds.idl._typesupport.DDS.XTypes.AnnotationTypeFlag', 'cyclonedds.idl._typesupport.DDS.XTypes.TypeFlag']
AliasTypeFlag = types.typedef['cyclonedds.idl._typesupport.DDS.XTypes.AliasTypeFlag', 'cyclonedds.idl._typesupport.DDS.XTypes.TypeFlag']
EnumTypeFlag = types.typedef['cyclonedds.idl._typesupport.DDS.XTypes.EnumTypeFlag', 'cyclonedds.idl._typesupport.DDS.XTypes.TypeFlag']
BitmaskTypeFlag = types.typedef['cyclonedds.idl._typesupport.DDS.XTypes.BitmaskTypeFlag', 'cyclonedds.idl._typesupport.DDS.XTypes.TypeFlag']
BitsetTypeFlag = types.typedef['cyclonedds.idl._typesupport.DDS.XTypes.BitsetTypeFlag', 'cyclonedds.idl._typesupport.DDS.XTypes.TypeFlag']
TypeFlagMinimalMask = 7

@dataclass
@annotate.final
@annotate.autoid("sequential")
@annotate.nested
class StringSTypeDefn(idl.IdlStruct, typename="DDS.XTypes.StringSTypeDefn"):
    bound: 'cyclonedds.idl._typesupport.DDS.XTypes.SBound'


@dataclass
@annotate.final
@annotate.autoid("sequential")
@annotate.nested
class StringLTypeDefn(idl.IdlStruct, typename="DDS.XTypes.StringLTypeDefn"):
    bound: 'cyclonedds.idl._typesupport.DDS.XTypes.LBound'


@dataclass
@annotate.final
@annotate.autoid("sequential")
@annotate.nested
class PlainCollectionHeader(idl.IdlStruct, typename="DDS.XTypes.PlainCollectionHeader"):
    equiv_kind: 'cyclonedds.idl._typesupport.DDS.XTypes.EquivalenceKind'
    element_flags: 'cyclonedds.idl._typesupport.DDS.XTypes.CollectionElementFlag'


@dataclass
@annotate.final
@annotate.autoid("sequential")
@annotate.nested
class PlainSequenceSElemDefn(idl.IdlStruct, typename="DDS.XTypes.PlainSequenceSElemDefn"):
    header: 'cyclonedds.idl._typesupport.DDS.XTypes.PlainCollectionHeader'
    bound: 'cyclonedds.idl._typesupport.DDS.XTypes.SBound'
    element_identifier: 'cyclonedds.idl._typesupport.DDS.XTypes.TypeIdentifier'
    annotate.external("element_identifier")


@dataclass
@annotate.final
@annotate.autoid("sequential")
@annotate.nested
class PlainSequenceLElemDefn(idl.IdlStruct, typename="DDS.XTypes.PlainSequenceLElemDefn"):
    header: 'cyclonedds.idl._typesupport.DDS.XTypes.PlainCollectionHeader'
    bound: 'cyclonedds.idl._typesupport.DDS.XTypes.LBound'
    element_identifier: 'cyclonedds.idl._typesupport.DDS.XTypes.TypeIdentifier'
    annotate.external("element_identifier")


@dataclass
@annotate.final
@annotate.autoid("sequential")
@annotate.nested
class PlainArraySElemDefn(idl.IdlStruct, typename="DDS.XTypes.PlainArraySElemDefn"):
    header: 'cyclonedds.idl._typesupport.DDS.XTypes.PlainCollectionHeader'
    array_bound_seq: 'cyclonedds.idl._typesupport.DDS.XTypes.SBoundSeq'
    element_identifier: 'cyclonedds.idl._typesupport.DDS.XTypes.TypeIdentifier'
    annotate.external("element_identifier")


@dataclass
@annotate.final
@annotate.autoid("sequential")
@annotate.nested
class PlainArrayLElemDefn(idl.IdlStruct, typename="DDS.XTypes.PlainArrayLElemDefn"):
    header: 'cyclonedds.idl._typesupport.DDS.XTypes.PlainCollectionHeader'
    array_bound_seq: 'cyclonedds.idl._typesupport.DDS.XTypes.LBoundSeq'
    element_identifier: 'cyclonedds.idl._typesupport.DDS.XTypes.TypeIdentifier'
    annotate.external("element_identifier")


@dataclass
@annotate.final
@annotate.autoid("sequential")
@annotate.nested
class PlainMapSTypeDefn(idl.IdlStruct, typename="DDS.XTypes.PlainMapSTypeDefn"):
    header: 'cyclonedds.idl._typesupport.DDS.XTypes.PlainCollectionHeader'
    bound: 'cyclonedds.idl._typesupport.DDS.XTypes.SBound'
    element_identifier: 'cyclonedds.idl._typesupport.DDS.XTypes.TypeIdentifier'
    annotate.external("element_identifier")
    key_flags: 'cyclonedds.idl._typesupport.DDS.XTypes.CollectionElementFlag'
    key_identifier: 'cyclonedds.idl._typesupport.DDS.XTypes.TypeIdentifier'
    annotate.external("key_identifier")


@dataclass
@annotate.final
@annotate.autoid("sequential")
@annotate.nested
class PlainMapLTypeDefn(idl.IdlStruct, typename="DDS.XTypes.PlainMapLTypeDefn"):
    header: 'cyclonedds.idl._typesupport.DDS.XTypes.PlainCollectionHeader'
    bound: 'cyclonedds.idl._typesupport.DDS.XTypes.LBound'
    element_identifier: 'cyclonedds.idl._typesupport.DDS.XTypes.TypeIdentifier'
    annotate.external("element_identifier")
    key_flags: 'cyclonedds.idl._typesupport.DDS.XTypes.CollectionElementFlag'
    key_identifier: 'cyclonedds.idl._typesupport.DDS.XTypes.TypeIdentifier'
    annotate.external("key_identifier")


@dataclass(unsafe_hash=True)
@annotate.appendable
@annotate.autoid("sequential")
@annotate.nested
class StronglyConnectedComponentId(idl.IdlStruct, typename="DDS.XTypes.StronglyConnectedComponentId"):
    sc_component_id: 'cyclonedds.idl._typesupport.DDS.XTypes.TypeObjectHashId'
    scc_length: types.int32
    scc_index: types.int32


@dataclass
@annotate.mutable
@annotate.autoid("sequential")
@annotate.nested
class ExtendedTypeDefn(idl.IdlStruct, typename="DDS.XTypes.ExtendedTypeDefn"):
    pass



@annotate.final
@annotate.nested
class TypeIdentifier(idl.IdlUnion, discriminator=types.byte, discriminator_is_key=False, typename="DDS.XTypes.TypeIdentifier"):
    string_sdefn: types.case[[112, 114], 'cyclonedds.idl._typesupport.DDS.XTypes.StringSTypeDefn']
    string_ldefn: types.case[[113, 115], 'cyclonedds.idl._typesupport.DDS.XTypes.StringLTypeDefn']
    seq_sdefn: types.case[[128], 'cyclonedds.idl._typesupport.DDS.XTypes.PlainSequenceSElemDefn']
    seq_ldefn: types.case[[129], 'cyclonedds.idl._typesupport.DDS.XTypes.PlainSequenceLElemDefn']
    array_sdefn: types.case[[144], 'cyclonedds.idl._typesupport.DDS.XTypes.PlainArraySElemDefn']
    array_ldefn: types.case[[145], 'cyclonedds.idl._typesupport.DDS.XTypes.PlainArrayLElemDefn']
    map_sdefn: types.case[[160], 'cyclonedds.idl._typesupport.DDS.XTypes.PlainMapSTypeDefn']
    map_ldefn: types.case[[161], 'cyclonedds.idl._typesupport.DDS.XTypes.PlainMapLTypeDefn']
    sc_component_id: types.case[[176], 'cyclonedds.idl._typesupport.DDS.XTypes.StronglyConnectedComponentId']
    equivalence_hash: types.case[[242, 241], 'cyclonedds.idl._typesupport.DDS.XTypes.EquivalenceHash']

TypeIdentifierSeq = types.typedef['cyclonedds.idl._typesupport.DDS.XTypes.TypeIdentifierSeq', types.sequence['cyclonedds.idl._typesupport.DDS.XTypes.TypeIdentifier']]
MemberId = types.typedef['cyclonedds.idl._typesupport.DDS.XTypes.MemberId', types.uint32]
ANNOTATION_STR_VALUE_MAX_LEN = 128
ANNOTATION_OCTETSEC_VALUE_MAX_LEN = 128

@dataclass
@annotate.mutable
@annotate.autoid("sequential")
@annotate.nested
class ExtendedAnnotationParameterValue(idl.IdlStruct, typename="DDS.XTypes.ExtendedAnnotationParameterValue"):
    pass



@annotate.final
@annotate.nested
class AnnotationParameterValue(idl.IdlUnion, discriminator=types.byte, discriminator_is_key=False, typename="DDS.XTypes.AnnotationParameterValue"):
    boolean_value: types.case[[1], bool]
    byte_value: types.case[[2], types.byte]
    int8_value: types.case[[12], types.int8]
    uint8_value: types.case[[13], types.uint8]
    int16_value: types.case[[3], types.int16]
    uint_16_value: types.case[[6], types.uint16]
    int32_value: types.case[[4], types.int32]
    uint32_value: types.case[[7], types.uint32]
    int64_value: types.case[[5], types.int64]
    uint64_value: types.case[[8], types.uint64]
    float32_value: types.case[[9], types.float32]
    float64_value: types.case[[10], types.float64]
    char_value: types.case[[16], types.char]
    enumerated_value: types.case[[64], types.int32]
    string8_value: types.case[[32], types.bounded_str[128]]


@dataclass
@annotate.appendable
@annotate.autoid("sequential")
@annotate.nested
class AppliedAnnotationParameter(idl.IdlStruct, typename="DDS.XTypes.AppliedAnnotationParameter"):
    paramname_hash: 'cyclonedds.idl._typesupport.DDS.XTypes.NameHash'
    value: 'cyclonedds.idl._typesupport.DDS.XTypes.AnnotationParameterValue'

AppliedAnnotationParameterSeq = types.typedef['cyclonedds.idl._typesupport.DDS.XTypes.AppliedAnnotationParameterSeq', types.sequence['cyclonedds.idl._typesupport.DDS.XTypes.AppliedAnnotationParameter']]

@dataclass
@annotate.appendable
@annotate.autoid("sequential")
@annotate.nested
class AppliedAnnotation(idl.IdlStruct, typename="DDS.XTypes.AppliedAnnotation"):
    annotation_typeid: 'cyclonedds.idl._typesupport.DDS.XTypes.TypeIdentifier'
    param_seq: Optional['cyclonedds.idl._typesupport.DDS.XTypes.AppliedAnnotationParameterSeq']

AppliedAnnotationSeq = types.typedef['cyclonedds.idl._typesupport.DDS.XTypes.AppliedAnnotationSeq', types.sequence['cyclonedds.idl._typesupport.DDS.XTypes.AppliedAnnotation']]

@dataclass
@annotate.final
@annotate.autoid("sequential")
@annotate.nested
class AppliedVerbatimAnnotation(idl.IdlStruct, typename="DDS.XTypes.AppliedVerbatimAnnotation"):
    placement: types.bounded_str[32]
    language: types.bounded_str[32]
    text: str


@dataclass
@annotate.appendable
@annotate.autoid("sequential")
@annotate.nested
class AppliedBuiltinMemberAnnotations(idl.IdlStruct, typename="DDS.XTypes.AppliedBuiltinMemberAnnotations"):
    unit: Optional[str]
    min: Optional['cyclonedds.idl._typesupport.DDS.XTypes.AnnotationParameterValue']
    max: Optional['cyclonedds.idl._typesupport.DDS.XTypes.AnnotationParameterValue']
    hash_id: Optional[str]


@dataclass
@annotate.final
@annotate.autoid("sequential")
@annotate.nested
class CommonStructMember(idl.IdlStruct, typename="DDS.XTypes.CommonStructMember"):
    member_id: 'cyclonedds.idl._typesupport.DDS.XTypes.MemberId'
    member_flags: 'cyclonedds.idl._typesupport.DDS.XTypes.StructMemberFlag'
    member_type_id: 'cyclonedds.idl._typesupport.DDS.XTypes.TypeIdentifier'


@dataclass
@annotate.final
@annotate.autoid("sequential")
@annotate.nested
class CompleteMemberDetail(idl.IdlStruct, typename="DDS.XTypes.CompleteMemberDetail"):
    name: 'cyclonedds.idl._typesupport.DDS.XTypes.MemberName'
    ann_builtin: Optional['cyclonedds.idl._typesupport.DDS.XTypes.AppliedBuiltinMemberAnnotations']
    ann_custom: Optional['cyclonedds.idl._typesupport.DDS.XTypes.AppliedAnnotationSeq']


@dataclass
@annotate.final
@annotate.autoid("sequential")
@annotate.nested
class MinimalMemberDetail(idl.IdlStruct, typename="DDS.XTypes.MinimalMemberDetail"):
    name_hash: 'cyclonedds.idl._typesupport.DDS.XTypes.NameHash'


@dataclass
@annotate.appendable
@annotate.autoid("sequential")
@annotate.nested
class CompleteStructMember(idl.IdlStruct, typename="DDS.XTypes.CompleteStructMember"):
    common: 'cyclonedds.idl._typesupport.DDS.XTypes.CommonStructMember'
    detail: 'cyclonedds.idl._typesupport.DDS.XTypes.CompleteMemberDetail'

CompleteStructMemberSeq = types.typedef['cyclonedds.idl._typesupport.DDS.XTypes.CompleteStructMemberSeq', types.sequence['cyclonedds.idl._typesupport.DDS.XTypes.CompleteStructMember']]

@dataclass
@annotate.appendable
@annotate.autoid("sequential")
@annotate.nested
class MinimalStructMember(idl.IdlStruct, typename="DDS.XTypes.MinimalStructMember"):
    common: 'cyclonedds.idl._typesupport.DDS.XTypes.CommonStructMember'
    detail: 'cyclonedds.idl._typesupport.DDS.XTypes.MinimalMemberDetail'

MinimalStructMemberSeq = types.typedef['cyclonedds.idl._typesupport.DDS.XTypes.MinimalStructMemberSeq', types.sequence['cyclonedds.idl._typesupport.DDS.XTypes.MinimalStructMember']]

@dataclass
@annotate.appendable
@annotate.autoid("sequential")
@annotate.nested
class AppliedBuiltinTypeAnnotations(idl.IdlStruct, typename="DDS.XTypes.AppliedBuiltinTypeAnnotations"):
    verbatim: Optional['cyclonedds.idl._typesupport.DDS.XTypes.AppliedVerbatimAnnotation']


@dataclass
@annotate.final
@annotate.autoid("sequential")
@annotate.nested
class MinimalTypeDetail(idl.IdlStruct, typename="DDS.XTypes.MinimalTypeDetail"):
    pass


@dataclass
@annotate.final
@annotate.autoid("sequential")
@annotate.nested
class CompleteTypeDetail(idl.IdlStruct, typename="DDS.XTypes.CompleteTypeDetail"):
    ann_builtin: Optional['cyclonedds.idl._typesupport.DDS.XTypes.AppliedBuiltinTypeAnnotations']
    ann_custom: Optional['cyclonedds.idl._typesupport.DDS.XTypes.AppliedAnnotationSeq']
    type_name: 'cyclonedds.idl._typesupport.DDS.XTypes.QualifiedTypeName'


@dataclass
@annotate.appendable
@annotate.autoid("sequential")
@annotate.nested
class CompleteStructHeader(idl.IdlStruct, typename="DDS.XTypes.CompleteStructHeader"):
    base_type: 'cyclonedds.idl._typesupport.DDS.XTypes.TypeIdentifier'
    detail: 'cyclonedds.idl._typesupport.DDS.XTypes.CompleteTypeDetail'


@dataclass
@annotate.appendable
@annotate.autoid("sequential")
@annotate.nested
class MinimalStructHeader(idl.IdlStruct, typename="DDS.XTypes.MinimalStructHeader"):
    base_type: 'cyclonedds.idl._typesupport.DDS.XTypes.TypeIdentifier'
    detail: 'cyclonedds.idl._typesupport.DDS.XTypes.MinimalTypeDetail'


@dataclass
@annotate.final
@annotate.autoid("sequential")
@annotate.nested
class CompleteStructType(idl.IdlStruct, typename="DDS.XTypes.CompleteStructType"):
    struct_flags: 'cyclonedds.idl._typesupport.DDS.XTypes.StructTypeFlag'
    header: 'cyclonedds.idl._typesupport.DDS.XTypes.CompleteStructHeader'
    member_seq: 'cyclonedds.idl._typesupport.DDS.XTypes.CompleteStructMemberSeq'


@dataclass
@annotate.final
@annotate.autoid("sequential")
@annotate.nested
class MinimalStructType(idl.IdlStruct, typename="DDS.XTypes.MinimalStructType"):
    struct_flags: 'cyclonedds.idl._typesupport.DDS.XTypes.StructTypeFlag'
    header: 'cyclonedds.idl._typesupport.DDS.XTypes.MinimalStructHeader'
    member_seq: 'cyclonedds.idl._typesupport.DDS.XTypes.MinimalStructMemberSeq'

UnionCaseLabelSeq = types.typedef['cyclonedds.idl._typesupport.DDS.XTypes.UnionCaseLabelSeq', types.sequence[types.int32]]

@dataclass
@annotate.final
@annotate.autoid("sequential")
@annotate.nested
class CommonUnionMember(idl.IdlStruct, typename="DDS.XTypes.CommonUnionMember"):
    member_id: 'cyclonedds.idl._typesupport.DDS.XTypes.MemberId'
    member_flags: 'cyclonedds.idl._typesupport.DDS.XTypes.UnionMemberFlag'
    type_id: 'cyclonedds.idl._typesupport.DDS.XTypes.TypeIdentifier'
    label_seq: 'cyclonedds.idl._typesupport.DDS.XTypes.UnionCaseLabelSeq'


@dataclass
@annotate.appendable
@annotate.autoid("sequential")
@annotate.nested
class CompleteUnionMember(idl.IdlStruct, typename="DDS.XTypes.CompleteUnionMember"):
    common: 'cyclonedds.idl._typesupport.DDS.XTypes.CommonUnionMember'
    detail: 'cyclonedds.idl._typesupport.DDS.XTypes.CompleteMemberDetail'

CompleteUnionMemberSeq = types.typedef['cyclonedds.idl._typesupport.DDS.XTypes.CompleteUnionMemberSeq', types.sequence['cyclonedds.idl._typesupport.DDS.XTypes.CompleteUnionMember']]

@dataclass
@annotate.appendable
@annotate.autoid("sequential")
@annotate.nested
class MinimalUnionMember(idl.IdlStruct, typename="DDS.XTypes.MinimalUnionMember"):
    common: 'cyclonedds.idl._typesupport.DDS.XTypes.CommonUnionMember'
    detail: 'cyclonedds.idl._typesupport.DDS.XTypes.MinimalMemberDetail'

MinimalUnionMemberSeq = types.typedef['cyclonedds.idl._typesupport.DDS.XTypes.MinimalUnionMemberSeq', types.sequence['cyclonedds.idl._typesupport.DDS.XTypes.MinimalUnionMember']]

@dataclass
@annotate.final
@annotate.autoid("sequential")
@annotate.nested
class CommonDiscriminatorMember(idl.IdlStruct, typename="DDS.XTypes.CommonDiscriminatorMember"):
    member_flags: 'cyclonedds.idl._typesupport.DDS.XTypes.UnionDiscriminatorFlag'
    type_id: 'cyclonedds.idl._typesupport.DDS.XTypes.TypeIdentifier'


@dataclass
@annotate.appendable
@annotate.autoid("sequential")
@annotate.nested
class CompleteDiscriminatorMember(idl.IdlStruct, typename="DDS.XTypes.CompleteDiscriminatorMember"):
    common: 'cyclonedds.idl._typesupport.DDS.XTypes.CommonDiscriminatorMember'
    ann_builtin: Optional['cyclonedds.idl._typesupport.DDS.XTypes.AppliedBuiltinTypeAnnotations']
    ann_custom: Optional['cyclonedds.idl._typesupport.DDS.XTypes.AppliedAnnotationSeq']


@dataclass
@annotate.appendable
@annotate.autoid("sequential")
@annotate.nested
class MinimalDiscriminatorMember(idl.IdlStruct, typename="DDS.XTypes.MinimalDiscriminatorMember"):
    common: 'cyclonedds.idl._typesupport.DDS.XTypes.CommonDiscriminatorMember'


@dataclass
@annotate.appendable
@annotate.autoid("sequential")
@annotate.nested
class CompleteUnionHeader(idl.IdlStruct, typename="DDS.XTypes.CompleteUnionHeader"):
    detail: 'cyclonedds.idl._typesupport.DDS.XTypes.CompleteTypeDetail'


@dataclass
@annotate.appendable
@annotate.autoid("sequential")
@annotate.nested
class MinimalUnionHeader(idl.IdlStruct, typename="DDS.XTypes.MinimalUnionHeader"):
    detail: 'cyclonedds.idl._typesupport.DDS.XTypes.MinimalTypeDetail'


@dataclass
@annotate.final
@annotate.autoid("sequential")
@annotate.nested
class CompleteUnionType(idl.IdlStruct, typename="DDS.XTypes.CompleteUnionType"):
    union_flags: 'cyclonedds.idl._typesupport.DDS.XTypes.UnionTypeFlag'
    header: 'cyclonedds.idl._typesupport.DDS.XTypes.CompleteUnionHeader'
    discriminator: 'cyclonedds.idl._typesupport.DDS.XTypes.CompleteDiscriminatorMember'
    member_seq: 'cyclonedds.idl._typesupport.DDS.XTypes.CompleteUnionMemberSeq'


@dataclass
@annotate.final
@annotate.autoid("sequential")
@annotate.nested
class MinimalUnionType(idl.IdlStruct, typename="DDS.XTypes.MinimalUnionType"):
    union_flags: 'cyclonedds.idl._typesupport.DDS.XTypes.UnionTypeFlag'
    header: 'cyclonedds.idl._typesupport.DDS.XTypes.MinimalUnionHeader'
    discriminator: 'cyclonedds.idl._typesupport.DDS.XTypes.MinimalDiscriminatorMember'
    member_seq: 'cyclonedds.idl._typesupport.DDS.XTypes.MinimalUnionMemberSeq'


@dataclass
@annotate.final
@annotate.autoid("sequential")
@annotate.nested
class CommonAnnotationParameter(idl.IdlStruct, typename="DDS.XTypes.CommonAnnotationParameter"):
    member_flags: 'cyclonedds.idl._typesupport.DDS.XTypes.AnnotationParameterFlag'
    member_type_id: 'cyclonedds.idl._typesupport.DDS.XTypes.TypeIdentifier'


@dataclass
@annotate.appendable
@annotate.autoid("sequential")
@annotate.nested
class CompleteAnnotationParameter(idl.IdlStruct, typename="DDS.XTypes.CompleteAnnotationParameter"):
    common: 'cyclonedds.idl._typesupport.DDS.XTypes.CommonAnnotationParameter'
    name: 'cyclonedds.idl._typesupport.DDS.XTypes.MemberName'
    default_value: 'cyclonedds.idl._typesupport.DDS.XTypes.AnnotationParameterValue'

CompleteAnnotationParameterSeq = types.typedef['cyclonedds.idl._typesupport.DDS.XTypes.CompleteAnnotationParameterSeq', types.sequence['cyclonedds.idl._typesupport.DDS.XTypes.CompleteAnnotationParameter']]

@dataclass
@annotate.appendable
@annotate.autoid("sequential")
@annotate.nested
class MinimalAnnotationParameter(idl.IdlStruct, typename="DDS.XTypes.MinimalAnnotationParameter"):
    common: 'cyclonedds.idl._typesupport.DDS.XTypes.CommonAnnotationParameter'
    name_hash: 'cyclonedds.idl._typesupport.DDS.XTypes.NameHash'
    default_value: 'cyclonedds.idl._typesupport.DDS.XTypes.AnnotationParameterValue'

MinimalAnnotationParameterSeq = types.typedef['cyclonedds.idl._typesupport.DDS.XTypes.MinimalAnnotationParameterSeq', types.sequence['cyclonedds.idl._typesupport.DDS.XTypes.MinimalAnnotationParameter']]

@dataclass
@annotate.appendable
@annotate.autoid("sequential")
@annotate.nested
class CompleteAnnotationHeader(idl.IdlStruct, typename="DDS.XTypes.CompleteAnnotationHeader"):
    annotation_name: 'cyclonedds.idl._typesupport.DDS.XTypes.QualifiedTypeName'


@dataclass
@annotate.appendable
@annotate.autoid("sequential")
@annotate.nested
class MinimalAnnotationHeader(idl.IdlStruct, typename="DDS.XTypes.MinimalAnnotationHeader"):
    pass


@dataclass
@annotate.final
@annotate.autoid("sequential")
@annotate.nested
class CompleteAnnotationType(idl.IdlStruct, typename="DDS.XTypes.CompleteAnnotationType"):
    annotation_flag: 'cyclonedds.idl._typesupport.DDS.XTypes.AnnotationTypeFlag'
    header: 'cyclonedds.idl._typesupport.DDS.XTypes.CompleteAnnotationHeader'
    member_seq: 'cyclonedds.idl._typesupport.DDS.XTypes.CompleteAnnotationParameterSeq'


@dataclass
@annotate.final
@annotate.autoid("sequential")
@annotate.nested
class MinimalAnnotationType(idl.IdlStruct, typename="DDS.XTypes.MinimalAnnotationType"):
    annotation_flag: 'cyclonedds.idl._typesupport.DDS.XTypes.AnnotationTypeFlag'
    header: 'cyclonedds.idl._typesupport.DDS.XTypes.MinimalAnnotationHeader'
    member_seq: 'cyclonedds.idl._typesupport.DDS.XTypes.MinimalAnnotationParameterSeq'


@dataclass
@annotate.final
@annotate.autoid("sequential")
@annotate.nested
class CommonAliasBody(idl.IdlStruct, typename="DDS.XTypes.CommonAliasBody"):
    related_flags: 'cyclonedds.idl._typesupport.DDS.XTypes.AliasMemberFlag'
    related_type: 'cyclonedds.idl._typesupport.DDS.XTypes.TypeIdentifier'


@dataclass
@annotate.appendable
@annotate.autoid("sequential")
@annotate.nested
class CompleteAliasBody(idl.IdlStruct, typename="DDS.XTypes.CompleteAliasBody"):
    common: 'cyclonedds.idl._typesupport.DDS.XTypes.CommonAliasBody'
    ann_builtin: Optional['cyclonedds.idl._typesupport.DDS.XTypes.AppliedBuiltinMemberAnnotations']
    ann_custom: Optional['cyclonedds.idl._typesupport.DDS.XTypes.AppliedAnnotationSeq']


@dataclass
@annotate.appendable
@annotate.autoid("sequential")
@annotate.nested
class MinimalAliasBody(idl.IdlStruct, typename="DDS.XTypes.MinimalAliasBody"):
    common: 'cyclonedds.idl._typesupport.DDS.XTypes.CommonAliasBody'


@dataclass
@annotate.appendable
@annotate.autoid("sequential")
@annotate.nested
class CompleteAliasHeader(idl.IdlStruct, typename="DDS.XTypes.CompleteAliasHeader"):
    detail: 'cyclonedds.idl._typesupport.DDS.XTypes.CompleteTypeDetail'


@dataclass
@annotate.appendable
@annotate.autoid("sequential")
@annotate.nested
class MinimalAliasHeader(idl.IdlStruct, typename="DDS.XTypes.MinimalAliasHeader"):
    pass


@dataclass
@annotate.final
@annotate.autoid("sequential")
@annotate.nested
class CompleteAliasType(idl.IdlStruct, typename="DDS.XTypes.CompleteAliasType"):
    alias_flags: 'cyclonedds.idl._typesupport.DDS.XTypes.AliasTypeFlag'
    header: 'cyclonedds.idl._typesupport.DDS.XTypes.CompleteAliasHeader'
    body: 'cyclonedds.idl._typesupport.DDS.XTypes.CompleteAliasBody'


@dataclass
@annotate.final
@annotate.autoid("sequential")
@annotate.nested
class MinimalAliasType(idl.IdlStruct, typename="DDS.XTypes.MinimalAliasType"):
    alias_flags: 'cyclonedds.idl._typesupport.DDS.XTypes.AliasTypeFlag'
    header: 'cyclonedds.idl._typesupport.DDS.XTypes.MinimalAliasHeader'
    body: 'cyclonedds.idl._typesupport.DDS.XTypes.MinimalAliasBody'


@dataclass
@annotate.final
@annotate.autoid("sequential")
@annotate.nested
class CompleteElementDetail(idl.IdlStruct, typename="DDS.XTypes.CompleteElementDetail"):
    ann_builtin: Optional['cyclonedds.idl._typesupport.DDS.XTypes.AppliedBuiltinMemberAnnotations']
    ann_custom: Optional['cyclonedds.idl._typesupport.DDS.XTypes.AppliedAnnotationSeq']


@dataclass
@annotate.final
@annotate.autoid("sequential")
@annotate.nested
class CommonCollectionElement(idl.IdlStruct, typename="DDS.XTypes.CommonCollectionElement"):
    element_flags: 'cyclonedds.idl._typesupport.DDS.XTypes.CollectionElementFlag'
    type: 'cyclonedds.idl._typesupport.DDS.XTypes.TypeIdentifier'


@dataclass
@annotate.appendable
@annotate.autoid("sequential")
@annotate.nested
class CompleteCollectionElement(idl.IdlStruct, typename="DDS.XTypes.CompleteCollectionElement"):
    common: 'cyclonedds.idl._typesupport.DDS.XTypes.CommonCollectionElement'
    detail: 'cyclonedds.idl._typesupport.DDS.XTypes.CompleteElementDetail'


@dataclass
@annotate.appendable
@annotate.autoid("sequential")
@annotate.nested
class MinimalCollectionElement(idl.IdlStruct, typename="DDS.XTypes.MinimalCollectionElement"):
    common: 'cyclonedds.idl._typesupport.DDS.XTypes.CommonCollectionElement'


@dataclass
@annotate.final
@annotate.autoid("sequential")
@annotate.nested
class CommonCollectionHeader(idl.IdlStruct, typename="DDS.XTypes.CommonCollectionHeader"):
    bound: 'cyclonedds.idl._typesupport.DDS.XTypes.LBound'


@dataclass
@annotate.appendable
@annotate.autoid("sequential")
@annotate.nested
class CompleteCollectionHeader(idl.IdlStruct, typename="DDS.XTypes.CompleteCollectionHeader"):
    common: 'cyclonedds.idl._typesupport.DDS.XTypes.CommonCollectionHeader'
    detail: Optional['cyclonedds.idl._typesupport.DDS.XTypes.CompleteTypeDetail']


@dataclass
@annotate.appendable
@annotate.autoid("sequential")
@annotate.nested
class MinimalCollectionHeader(idl.IdlStruct, typename="DDS.XTypes.MinimalCollectionHeader"):
    common: 'cyclonedds.idl._typesupport.DDS.XTypes.CommonCollectionHeader'


@dataclass
@annotate.final
@annotate.autoid("sequential")
@annotate.nested
class CompleteSequenceType(idl.IdlStruct, typename="DDS.XTypes.CompleteSequenceType"):
    collection_flag: 'cyclonedds.idl._typesupport.DDS.XTypes.CollectionTypeFlag'
    header: 'cyclonedds.idl._typesupport.DDS.XTypes.CompleteCollectionHeader'
    element: 'cyclonedds.idl._typesupport.DDS.XTypes.CompleteCollectionElement'


@dataclass
@annotate.final
@annotate.autoid("sequential")
@annotate.nested
class MinimalSequenceType(idl.IdlStruct, typename="DDS.XTypes.MinimalSequenceType"):
    collection_flag: 'cyclonedds.idl._typesupport.DDS.XTypes.CollectionTypeFlag'
    header: 'cyclonedds.idl._typesupport.DDS.XTypes.MinimalCollectionHeader'
    element: 'cyclonedds.idl._typesupport.DDS.XTypes.MinimalCollectionElement'


@dataclass
@annotate.final
@annotate.autoid("sequential")
@annotate.nested
class CommonArrayHeader(idl.IdlStruct, typename="DDS.XTypes.CommonArrayHeader"):
    bound_seq: 'cyclonedds.idl._typesupport.DDS.XTypes.LBoundSeq'


@dataclass
@annotate.appendable
@annotate.autoid("sequential")
@annotate.nested
class CompleteArrayHeader(idl.IdlStruct, typename="DDS.XTypes.CompleteArrayHeader"):
    common: 'cyclonedds.idl._typesupport.DDS.XTypes.CommonArrayHeader'
    detail: 'cyclonedds.idl._typesupport.DDS.XTypes.CompleteTypeDetail'


@dataclass
@annotate.appendable
@annotate.autoid("sequential")
@annotate.nested
class MinimalArrayHeader(idl.IdlStruct, typename="DDS.XTypes.MinimalArrayHeader"):
    common: 'cyclonedds.idl._typesupport.DDS.XTypes.CommonArrayHeader'


@dataclass
@annotate.appendable
@annotate.autoid("sequential")
@annotate.nested
class CompleteArrayType(idl.IdlStruct, typename="DDS.XTypes.CompleteArrayType"):
    collection_flag: 'cyclonedds.idl._typesupport.DDS.XTypes.CollectionTypeFlag'
    header: 'cyclonedds.idl._typesupport.DDS.XTypes.CompleteArrayHeader'
    element: 'cyclonedds.idl._typesupport.DDS.XTypes.CompleteCollectionElement'


@dataclass
@annotate.final
@annotate.autoid("sequential")
@annotate.nested
class MinimalArrayType(idl.IdlStruct, typename="DDS.XTypes.MinimalArrayType"):
    collection_flag: 'cyclonedds.idl._typesupport.DDS.XTypes.CollectionTypeFlag'
    header: 'cyclonedds.idl._typesupport.DDS.XTypes.MinimalArrayHeader'
    element: 'cyclonedds.idl._typesupport.DDS.XTypes.MinimalCollectionElement'


@dataclass
@annotate.final
@annotate.autoid("sequential")
@annotate.nested
class CompleteMapType(idl.IdlStruct, typename="DDS.XTypes.CompleteMapType"):
    collection_flag: 'cyclonedds.idl._typesupport.DDS.XTypes.CollectionTypeFlag'
    header: 'cyclonedds.idl._typesupport.DDS.XTypes.CompleteCollectionHeader'
    key: 'cyclonedds.idl._typesupport.DDS.XTypes.CompleteCollectionElement'
    element: 'cyclonedds.idl._typesupport.DDS.XTypes.CompleteCollectionElement'


@dataclass
@annotate.final
@annotate.autoid("sequential")
@annotate.nested
class MinimalMapType(idl.IdlStruct, typename="DDS.XTypes.MinimalMapType"):
    collection_flag: 'cyclonedds.idl._typesupport.DDS.XTypes.CollectionTypeFlag'
    header: 'cyclonedds.idl._typesupport.DDS.XTypes.MinimalCollectionHeader'
    key: 'cyclonedds.idl._typesupport.DDS.XTypes.MinimalCollectionElement'
    element: 'cyclonedds.idl._typesupport.DDS.XTypes.MinimalCollectionElement'

BitBound = types.typedef['cyclonedds.idl._typesupport.DDS.XTypes.BitBound', types.uint16]

@dataclass
@annotate.appendable
@annotate.autoid("sequential")
@annotate.nested
class CommonEnumeratedLiteral(idl.IdlStruct, typename="DDS.XTypes.CommonEnumeratedLiteral"):
    value: types.int32
    flags: 'cyclonedds.idl._typesupport.DDS.XTypes.EnumeratedLiteralFlag'


@dataclass
@annotate.appendable
@annotate.autoid("sequential")
@annotate.nested
class CompleteEnumeratedLiteral(idl.IdlStruct, typename="DDS.XTypes.CompleteEnumeratedLiteral"):
    common: 'cyclonedds.idl._typesupport.DDS.XTypes.CommonEnumeratedLiteral'
    detail: 'cyclonedds.idl._typesupport.DDS.XTypes.CompleteMemberDetail'

CompleteEnumeratedLiteralSeq = types.typedef['cyclonedds.idl._typesupport.DDS.XTypes.CompleteEnumeratedLiteralSeq', types.sequence['cyclonedds.idl._typesupport.DDS.XTypes.CompleteEnumeratedLiteral']]

@dataclass
@annotate.appendable
@annotate.autoid("sequential")
@annotate.nested
class MinimalEnumeratedLiteral(idl.IdlStruct, typename="DDS.XTypes.MinimalEnumeratedLiteral"):
    common: 'cyclonedds.idl._typesupport.DDS.XTypes.CommonEnumeratedLiteral'
    detail: 'cyclonedds.idl._typesupport.DDS.XTypes.MinimalMemberDetail'

MinimalEnumeratedLiteralSeq = types.typedef['cyclonedds.idl._typesupport.DDS.XTypes.MinimalEnumeratedLiteralSeq', types.sequence['cyclonedds.idl._typesupport.DDS.XTypes.MinimalEnumeratedLiteral']]

@dataclass
@annotate.final
@annotate.autoid("sequential")
@annotate.nested
class CommonEnumeratedHeader(idl.IdlStruct, typename="DDS.XTypes.CommonEnumeratedHeader"):
    bit_bound: 'cyclonedds.idl._typesupport.DDS.XTypes.BitBound'


@dataclass
@annotate.appendable
@annotate.autoid("sequential")
@annotate.nested
class CompleteEnumeratedHeader(idl.IdlStruct, typename="DDS.XTypes.CompleteEnumeratedHeader"):
    common: 'cyclonedds.idl._typesupport.DDS.XTypes.CommonEnumeratedHeader'
    detail: 'cyclonedds.idl._typesupport.DDS.XTypes.CompleteTypeDetail'


@dataclass
@annotate.appendable
@annotate.autoid("sequential")
@annotate.nested
class MinimalEnumeratedHeader(idl.IdlStruct, typename="DDS.XTypes.MinimalEnumeratedHeader"):
    common: 'cyclonedds.idl._typesupport.DDS.XTypes.CommonEnumeratedHeader'


@dataclass
@annotate.final
@annotate.autoid("sequential")
@annotate.nested
class CompleteEnumeratedType(idl.IdlStruct, typename="DDS.XTypes.CompleteEnumeratedType"):
    enum_flags: 'cyclonedds.idl._typesupport.DDS.XTypes.EnumTypeFlag'
    header: 'cyclonedds.idl._typesupport.DDS.XTypes.CompleteEnumeratedHeader'
    literal_seq: 'cyclonedds.idl._typesupport.DDS.XTypes.CompleteEnumeratedLiteralSeq'


@dataclass
@annotate.final
@annotate.autoid("sequential")
@annotate.nested
class MinimalEnumeratedType(idl.IdlStruct, typename="DDS.XTypes.MinimalEnumeratedType"):
    enum_flags: 'cyclonedds.idl._typesupport.DDS.XTypes.EnumTypeFlag'
    header: 'cyclonedds.idl._typesupport.DDS.XTypes.MinimalEnumeratedHeader'
    literal_seq: 'cyclonedds.idl._typesupport.DDS.XTypes.MinimalEnumeratedLiteralSeq'


@dataclass
@annotate.final
@annotate.autoid("sequential")
@annotate.nested
class CommonBitflag(idl.IdlStruct, typename="DDS.XTypes.CommonBitflag"):
    position: types.uint16
    flags: 'cyclonedds.idl._typesupport.DDS.XTypes.BitflagFlag'


@dataclass
@annotate.appendable
@annotate.autoid("sequential")
@annotate.nested
class CompleteBitflag(idl.IdlStruct, typename="DDS.XTypes.CompleteBitflag"):
    common: 'cyclonedds.idl._typesupport.DDS.XTypes.CommonBitflag'
    detail: 'cyclonedds.idl._typesupport.DDS.XTypes.CompleteMemberDetail'

CompleteBitflagSeq = types.typedef['cyclonedds.idl._typesupport.DDS.XTypes.CompleteBitflagSeq', types.sequence['cyclonedds.idl._typesupport.DDS.XTypes.CompleteBitflag']]

@dataclass
@annotate.appendable
@annotate.autoid("sequential")
@annotate.nested
class MinimalBitflag(idl.IdlStruct, typename="DDS.XTypes.MinimalBitflag"):
    common: 'cyclonedds.idl._typesupport.DDS.XTypes.CommonBitflag'
    detail: 'cyclonedds.idl._typesupport.DDS.XTypes.MinimalMemberDetail'

MinimalBitflagSeq = types.typedef['cyclonedds.idl._typesupport.DDS.XTypes.MinimalBitflagSeq', types.sequence['cyclonedds.idl._typesupport.DDS.XTypes.MinimalBitflag']]

@dataclass
@annotate.final
@annotate.autoid("sequential")
@annotate.nested
class CommonBitmaskHeader(idl.IdlStruct, typename="DDS.XTypes.CommonBitmaskHeader"):
    bit_bound: 'cyclonedds.idl._typesupport.DDS.XTypes.BitBound'

CompleteBitmaskHeader = types.typedef['cyclonedds.idl._typesupport.DDS.XTypes.CompleteBitmaskHeader', 'cyclonedds.idl._typesupport.DDS.XTypes.CompleteEnumeratedHeader']
MinimalBitmaskHeader = types.typedef['cyclonedds.idl._typesupport.DDS.XTypes.MinimalBitmaskHeader', 'cyclonedds.idl._typesupport.DDS.XTypes.MinimalEnumeratedHeader']

@dataclass
@annotate.appendable
@annotate.autoid("sequential")
@annotate.nested
class CompleteBitmaskType(idl.IdlStruct, typename="DDS.XTypes.CompleteBitmaskType"):
    bitmask_flags: 'cyclonedds.idl._typesupport.DDS.XTypes.BitmaskTypeFlag'
    header: 'cyclonedds.idl._typesupport.DDS.XTypes.CompleteBitmaskHeader'
    flag_seq: 'cyclonedds.idl._typesupport.DDS.XTypes.CompleteBitflagSeq'


@dataclass
@annotate.appendable
@annotate.autoid("sequential")
@annotate.nested
class MinimalBitmaskType(idl.IdlStruct, typename="DDS.XTypes.MinimalBitmaskType"):
    bitmask_flags: 'cyclonedds.idl._typesupport.DDS.XTypes.BitmaskTypeFlag'
    header: 'cyclonedds.idl._typesupport.DDS.XTypes.MinimalBitmaskHeader'
    flag_seq: 'cyclonedds.idl._typesupport.DDS.XTypes.MinimalBitflagSeq'


@dataclass
@annotate.final
@annotate.autoid("sequential")
@annotate.nested
class CommonBitfield(idl.IdlStruct, typename="DDS.XTypes.CommonBitfield"):
    position: types.uint16
    flags: 'cyclonedds.idl._typesupport.DDS.XTypes.BitsetMemberFlag'
    bitcount: types.byte
    holder_type: 'cyclonedds.idl._typesupport.DDS.XTypes.TypeKind'


@dataclass
@annotate.appendable
@annotate.autoid("sequential")
@annotate.nested
class CompleteBitfield(idl.IdlStruct, typename="DDS.XTypes.CompleteBitfield"):
    common: 'cyclonedds.idl._typesupport.DDS.XTypes.CommonBitfield'
    detail: 'cyclonedds.idl._typesupport.DDS.XTypes.CompleteMemberDetail'

CompleteBitfieldSeq = types.typedef['cyclonedds.idl._typesupport.DDS.XTypes.CompleteBitfieldSeq', types.sequence['cyclonedds.idl._typesupport.DDS.XTypes.CompleteBitfield']]

@dataclass
@annotate.appendable
@annotate.autoid("sequential")
@annotate.nested
class MinimalBitfield(idl.IdlStruct, typename="DDS.XTypes.MinimalBitfield"):
    common: 'cyclonedds.idl._typesupport.DDS.XTypes.CommonBitfield'
    name_hash: 'cyclonedds.idl._typesupport.DDS.XTypes.NameHash'

MinimalBitfieldSeq = types.typedef['cyclonedds.idl._typesupport.DDS.XTypes.MinimalBitfieldSeq', types.sequence['cyclonedds.idl._typesupport.DDS.XTypes.MinimalBitfield']]

@dataclass
@annotate.appendable
@annotate.autoid("sequential")
@annotate.nested
class CompleteBitsetHeader(idl.IdlStruct, typename="DDS.XTypes.CompleteBitsetHeader"):
    detail: 'cyclonedds.idl._typesupport.DDS.XTypes.CompleteTypeDetail'


@dataclass
@annotate.appendable
@annotate.autoid("sequential")
@annotate.nested
class MinimalBitsetHeader(idl.IdlStruct, typename="DDS.XTypes.MinimalBitsetHeader"):
    pass


@dataclass
@annotate.appendable
@annotate.autoid("sequential")
@annotate.nested
class CompleteBitsetType(idl.IdlStruct, typename="DDS.XTypes.CompleteBitsetType"):
    bitset_flags: 'cyclonedds.idl._typesupport.DDS.XTypes.BitsetTypeFlag'
    header: 'cyclonedds.idl._typesupport.DDS.XTypes.CompleteBitsetHeader'
    field_seq: 'cyclonedds.idl._typesupport.DDS.XTypes.CompleteBitfieldSeq'


@dataclass
@annotate.appendable
@annotate.autoid("sequential")
@annotate.nested
class MinimalBitsetType(idl.IdlStruct, typename="DDS.XTypes.MinimalBitsetType"):
    bitset_flags: 'cyclonedds.idl._typesupport.DDS.XTypes.BitsetTypeFlag'
    header: 'cyclonedds.idl._typesupport.DDS.XTypes.MinimalBitsetHeader'
    field_seq: 'cyclonedds.idl._typesupport.DDS.XTypes.MinimalBitfieldSeq'


@dataclass
@annotate.mutable
@annotate.autoid("sequential")
@annotate.nested
class CompleteExtendedType(idl.IdlStruct, typename="DDS.XTypes.CompleteExtendedType"):
    pass



@annotate.final
@annotate.nested
class CompleteTypeObject(idl.IdlUnion, discriminator=types.byte, discriminator_is_key=False, typename="DDS.XTypes.CompleteTypeObject"):
    alias_type: types.case[[48], 'cyclonedds.idl._typesupport.DDS.XTypes.CompleteAliasType']
    annotation_type: types.case[[80], 'cyclonedds.idl._typesupport.DDS.XTypes.CompleteAnnotationType']
    struct_type: types.case[[81], 'cyclonedds.idl._typesupport.DDS.XTypes.CompleteStructType']
    union_type: types.case[[82], 'cyclonedds.idl._typesupport.DDS.XTypes.CompleteUnionType']
    bitset_type: types.case[[83], 'cyclonedds.idl._typesupport.DDS.XTypes.CompleteBitsetType']
    sequence_type: types.case[[96], 'cyclonedds.idl._typesupport.DDS.XTypes.CompleteSequenceType']
    array_type: types.case[[97], 'cyclonedds.idl._typesupport.DDS.XTypes.CompleteArrayType']
    map_type: types.case[[98], 'cyclonedds.idl._typesupport.DDS.XTypes.CompleteMapType']
    enumerated_type: types.case[[64], 'cyclonedds.idl._typesupport.DDS.XTypes.CompleteEnumeratedType']
    bitmask_type: types.case[[65], 'cyclonedds.idl._typesupport.DDS.XTypes.CompleteBitmaskType']


@dataclass
@annotate.mutable
@annotate.autoid("sequential")
@annotate.nested
class MinimalExtendedType(idl.IdlStruct, typename="DDS.XTypes.MinimalExtendedType"):
    pass



@annotate.final
@annotate.nested
class MinimalTypeObject(idl.IdlUnion, discriminator=types.byte, discriminator_is_key=False, typename="DDS.XTypes.MinimalTypeObject"):
    alias_type: types.case[[48], 'cyclonedds.idl._typesupport.DDS.XTypes.MinimalAliasType']
    annotation_type: types.case[[80], 'cyclonedds.idl._typesupport.DDS.XTypes.MinimalAnnotationType']
    struct_type: types.case[[81], 'cyclonedds.idl._typesupport.DDS.XTypes.MinimalStructType']
    union_type: types.case[[82], 'cyclonedds.idl._typesupport.DDS.XTypes.MinimalUnionType']
    bitset_type: types.case[[83], 'cyclonedds.idl._typesupport.DDS.XTypes.MinimalBitsetType']
    sequence_type: types.case[[96], 'cyclonedds.idl._typesupport.DDS.XTypes.MinimalSequenceType']
    array_type: types.case[[97], 'cyclonedds.idl._typesupport.DDS.XTypes.MinimalArrayType']
    map_type: types.case[[98], 'cyclonedds.idl._typesupport.DDS.XTypes.MinimalMapType']
    enumerated_type: types.case[[64], 'cyclonedds.idl._typesupport.DDS.XTypes.MinimalEnumeratedType']
    bitmask_type: types.case[[65], 'cyclonedds.idl._typesupport.DDS.XTypes.MinimalBitmaskType']



@annotate.appendable
@annotate.nested
class TypeObject(idl.IdlUnion, discriminator=types.byte, discriminator_is_key=False, typename="DDS.XTypes.TypeObject"):
    complete: types.case[[242], 'cyclonedds.idl._typesupport.DDS.XTypes.CompleteTypeObject']
    minimal: types.case[[241], 'cyclonedds.idl._typesupport.DDS.XTypes.MinimalTypeObject']

TypeObjectSeq = types.typedef['cyclonedds.idl._typesupport.DDS.XTypes.TypeObjectSeq', types.sequence['cyclonedds.idl._typesupport.DDS.XTypes.TypeObject']]
StronglyConnectedComponent = types.typedef['cyclonedds.idl._typesupport.DDS.XTypes.StronglyConnectedComponent', 'cyclonedds.idl._typesupport.DDS.XTypes.TypeObjectSeq']

@dataclass
@annotate.final
@annotate.autoid("sequential")
@annotate.nested
class TypeIdentifierTypeObjectPair(idl.IdlStruct, typename="DDS.XTypes.TypeIdentifierTypeObjectPair"):
    type_identifier: 'cyclonedds.idl._typesupport.DDS.XTypes.TypeIdentifier'
    type_object: 'cyclonedds.idl._typesupport.DDS.XTypes.TypeObject'

TypeIdentifierTypeObjectPairSeq = types.typedef['cyclonedds.idl._typesupport.DDS.XTypes.TypeIdentifierTypeObjectPairSeq', types.sequence['cyclonedds.idl._typesupport.DDS.XTypes.TypeIdentifierTypeObjectPair']]

@dataclass
@annotate.final
@annotate.autoid("sequential")
@annotate.nested
class TypeIdentifierPair(idl.IdlStruct, typename="DDS.XTypes.TypeIdentifierPair"):
    type_identifier1: 'cyclonedds.idl._typesupport.DDS.XTypes.TypeIdentifier'
    type_identifier2: 'cyclonedds.idl._typesupport.DDS.XTypes.TypeIdentifier'

TypeIdentifierPairSeq = types.typedef['cyclonedds.idl._typesupport.DDS.XTypes.TypeIdentifierPairSeq', types.sequence['cyclonedds.idl._typesupport.DDS.XTypes.TypeIdentifierPair']]

@dataclass
@annotate.appendable
@annotate.autoid("sequential")
@annotate.nested
class TypeIdentifierWithSize(idl.IdlStruct, typename="DDS.XTypes.TypeIdentifierWithSize"):
    type_id: 'cyclonedds.idl._typesupport.DDS.XTypes.TypeIdentifier'
    typeobject_serialized_size: types.uint32

TypeIdentfierWithSizeSeq = types.typedef['cyclonedds.idl._typesupport.DDS.XTypes.TypeIdentfierWithSizeSeq', types.sequence['cyclonedds.idl._typesupport.DDS.XTypes.TypeIdentifierWithSize']]

@dataclass
@annotate.appendable
@annotate.autoid("sequential")
@annotate.nested
class TypeIdentifierWithDependencies(idl.IdlStruct, typename="DDS.XTypes.TypeIdentifierWithDependencies"):
    typeid_with_size: 'cyclonedds.idl._typesupport.DDS.XTypes.TypeIdentifierWithSize'
    dependent_typeid_count: types.int32
    dependent_typeids: types.sequence['cyclonedds.idl._typesupport.DDS.XTypes.TypeIdentifierWithSize']

TypeIdentifierWithDependenciesSeq = types.typedef['cyclonedds.idl._typesupport.DDS.XTypes.TypeIdentifierWithDependenciesSeq', types.sequence['cyclonedds.idl._typesupport.DDS.XTypes.TypeIdentifierWithDependencies']]

@dataclass
@annotate.mutable
@annotate.autoid("sequential")
@annotate.nested
class TypeInformation(idl.IdlStruct, typename="DDS.XTypes.TypeInformation"):
    minimal: 'cyclonedds.idl._typesupport.DDS.XTypes.TypeIdentifierWithDependencies'
    annotate.member_id("minimal", 4097)
    complete: 'cyclonedds.idl._typesupport.DDS.XTypes.TypeIdentifierWithDependencies'
    annotate.member_id("complete", 4098)

TypeInformationSeq = types.typedef['cyclonedds.idl._typesupport.DDS.XTypes.TypeInformationSeq', types.sequence['cyclonedds.idl._typesupport.DDS.XTypes.TypeInformation']]




import rich_click as click
from rich.syntax import Syntax
from rich.console import Console

from .layout.ping import make_ping_layout, make_ping_updater
from .layout.pong import make_pong_layout, make_pong_updater
from .layout.subscribe import make_sub_layout, make_sub_updater
from .layout.publish import make_pub_layout, make_pub_updater
from .layout.app import DDSPerfApp
from .utils import TimeDeltaParamType, RateParamType, SizeParamType


@click.group()
@click.option(
    "-T",
    "--topic",
    default="KS",
    type=click.Choice(
        ["KS", "K32", "K256", "OU", "UK16", "UK1024", "S16", "S256", "S4k", "S32k"]
    ),
    help="""Topic (KS is default). Print topic information with [bold yellow]cyclonedds performance topics[/]""",
)
@click.option(
    "-n",
    "--num-keys",
    type=int,
    help="Number of key values to use for data (only for topics with a key value)",
)
@click.option(
    "-u", "--unreliable", is_flag=True, help="Use best-effort instead of reliable"
)
@click.option(
    "-k",
    "--keep",
    metavar="<all,n>",
    help="Keep-all or keep-last-N for data (ping/pong is always keep-last-1)",
)
@click.option(
    "-c", "--cpu", is_flag=True, help="Subscribe to CPU stats from peers and show them"
)
@click.option(
    "-d",
    "--device-load",
    metavar="<device:bandwidth>",
    help="Report network load for device DEV with nominal bandwidth BW in bits/s (e.g., eth0:1e9)",
)
@click.option(
    "-D",
    "--duration",
    type=TimeDeltaParamType(),
    help="Set max duration, integer number of seconds or timestring like '1h23m11s'",
)
@click.option(
    "-L",
    "--local-matching",
    is_flag=True,
    help="Allow matching with endpoints in the same process to get throughput/latency in the same ddsperf process",
)
@click.option(
    "-Q",
    "--success-criterion",
    multiple=True,
    help="""Set success criteria
  [bold yellow]rss:X%[/] max allowed increase in RSS, in %,
  [bold yellow]rss:X[/] max allowed increase in RSS, in MB,
  [bold yellow]samples:N[/] min received messages by "sub",
  [bold yellow]roundtrips:N[/] min roundtrips for "pong",
  [bold yellow]minmatch:N[/] require >= N matching participants,
  [bold yellow]initwait:DUR[/] wait for those participants before starting, abort if not within DUR seconds,
  [bold yellow]maxwait:DUR[/] require those participants to match within DUR seconds,
""",
)
@click.option(
    "-R",
    "--reference-time",
    metavar="<TREF>",
    help="Show timestamps in the output relative to TREF instead of process start",
)
@click.option(
    "-W",
    "--wait-match-max",
    type=TimeDeltaParamType(),
    help="""
Maximum waittime for the minimum required number of matching participants (set by [bold green]-Qminmatch:N[/])
to show up before starting reading/writing data, terminate with an error otherwise. This differs
from [bold green]-Qmaxwait:DUR[/] because that doesn't delay starting and doesn't terminate the process before doing
anything""",
)
@click.option(
    "-i",
    "--domain-id",
    "--id",
    metavar="<ID>",
    type=int,
    help="Use domain ID instead of the default domain",
)
@click.option(
    "--color",
    type=click.Choice(["auto", "standard", "256", "truecolor", "windows", "none"]),
    default="auto",
    help="""Force the command to output with/without terminal colors. By default output colours if the terminal supports it."
See the [underline blue][link=https://rich.readthedocs.io/en/stable/console.html#color-systems]Rich documentation[/link][/] for more info on what the options mean.""",
)
@click.option(
    # This option is used for the sake of documentation image generation.
    "--render-output-once-on-exit",
    hidden=True,
    is_flag=True,
)
@click.pass_context
def performance(
    ctx,
    local_matching,
    topic,
    num_keys,
    unreliable,
    keep,
    cpu,
    device_load,
    duration,
    success_criterion,
    reference_time,
    wait_match_max,
    domain_id,
    color,
    render_output_once_on_exit,
):
    """Run CycloneDDS performance tests on your system.\b

    [bold blue]Examples[/]

    [bold][bright_yellow]cyclonedds performance publish[/] [green]--size[/] [red]1000[/]
    [bright_yellow]cyclonedds performance subscribe[/][/]

    -   [i]basic throughput test with 1024-byte samples[/]

    [bold][bright_yellow]cyclonedds performance ping
    cyclonedds performance pong[/][/]

    -   [i]basic latency test[/]
    """

    cmd = ["ddsperf", "-1", "-X"]

    if local_matching:
        cmd.append("-L")

    if topic is not None:
        cmd += ["-T", topic]

    if num_keys is not None:
        cmd += ["-n", f"{num_keys}"]

    if unreliable:
        cmd.append("-u")

    if keep is not None:
        cmd += ["-k", keep]

    if cpu:
        cmd.append("-c")

    if device_load is not None:
        cmd += ["-c", device_load]

    if duration is not None:
        cmd += ["-D", f"{duration.total_seconds()}"]

    for sct in success_criterion:
        cmd += [f"-Q{sct}"]

    if reference_time is not None:
        cmd += ["-R", reference_time]

    if wait_match_max is not None:
        cmd += ["-W", f"{wait_match_max.total_seconds()}"]

    if domain_id is not None:
        cmd += ["-i", str(domain_id)]

    ctx.ensure_object(dict)
    ctx.obj["ddsperf"] = cmd
    ctx.obj["color"] = color
    ctx.obj["render_output_once"] = render_output_once_on_exit


@performance.command("ping", short_help="Send pings")
@click.option(
    "-r", "--rate", type=RateParamType(), help="The rate at which to send pings"
)
@click.option(
    "-s",
    "--size",
    type=SizeParamType(),
    help="May be set if the topic is KS, specifies the (total) payload size in bytes",
)
@click.option(
    "-m",
    "--triggering-mode",
    default="listener",
    type=click.Choice(["listener", "waitset"]),
)
@click.pass_context
def ping(ctx, rate, size, triggering_mode):
    """Send a ping upon receiving all expected pongs, or send a ping at
    rate R (optionally suffixed with Hz/kHz). The triggering mode is either
    a listener (default, unless [i]local matching[/] has been specified) or a waitset."""

    cmd = ["ping"]

    if rate is not None:
        cmd += [f"{rate}"]

    if size is not None:
        cmd += ["size", f"{size}"]

    if triggering_mode:
        cmd += [triggering_mode]

    DDSPerfApp(
        Console(color_system=None if ctx.obj["color"] == "none" else ctx.obj["color"]),
        ctx.obj["ddsperf"] + cmd,
        make_ping_layout,
        make_ping_updater,
    ).run(ctx.obj["render_output_once"])


@performance.command("pong", short_help="Send pongs")
@click.option(
    "-m",
    "--triggering-mode",
    default="listener",
    type=click.Choice(["listener", "waitset"]),
)
@click.pass_context
def pong(ctx, triggering_mode):
    """A "dummy" mode that serves two purposes: configuring the triggering.
    mode (but it is shared with ping's mode), and suppressing the 1Hz ping
    if no other options are selected.  It always responds to pings."""

    cmd = ["pong"]

    if triggering_mode:
        cmd += [triggering_mode]

    DDSPerfApp(
        Console(color_system=None if ctx.obj["color"] == "none" else ctx.obj["color"]),
        ctx.obj["ddsperf"] + cmd,
        make_pong_layout,
        make_pong_updater,
    ).run(ctx.obj["render_output_once"])


@performance.command("subscribe", short_help="Subscribe to data")
@click.option(
    "-m",
    "--triggering-mode",
    default="listener",
    type=click.Choice(["listener", "waitset", "polling"]),
)
@click.pass_context
def subscribe(ctx, triggering_mode):
    """Subscribe to data, with calls to take occurring either in a listener
    (default), when a waitset is triggered, or by polling at 1kHz."""

    cmd = ["sub"]

    if triggering_mode:
        cmd += [triggering_mode]

    DDSPerfApp(
        Console(color_system=None if ctx.obj["color"] == "none" else ctx.obj["color"]),
        ctx.obj["ddsperf"] + cmd,
        make_sub_layout,
        make_sub_updater,
    ).run(ctx.obj["render_output_once"])


@performance.command("publish", short_help="Publish data")
@click.option("-r", "--rate", type=RateParamType(), help="The rate at which to publish")
@click.option(
    "-s",
    "--size",
    type=SizeParamType(),
    help="May be set if the topic is KS, specifies the (total) payload size in bytes",
)
@click.option("-b", "--burst", type=int)
@click.option("-p", "--ping", type=int)
@click.pass_context
def pub(ctx, rate, size, burst, ping):
    """Publish bursts of data at a set rate or as fast as possible if unspecified.
    Each burst is a single sample by default, but can be set to larger value using
    "--burst N". Sample size is controlled using the size parameter when the KS topic
    is used, minimal size is used when unspecified.
    If desired, a fraction of the samples can be treated as if it were a
    ping, for this, specify a percentage with "--ping 50" for fifty percent."""

    cmd = ["pub"]

    if rate is not None:
        cmd += [f"{rate}"]

    if size is not None:
        cmd += ["size", f"{size}"]

    if burst is not None:
        cmd += ["burst", f"{burst}"]

    if ping is not None:
        cmd += ["ping", f"{ping}%"]

    DDSPerfApp(
        Console(color_system=None if ctx.obj["color"] == "none" else ctx.obj["color"]),
        ctx.obj["ddsperf"] + cmd,
        make_pub_layout,
        make_pub_updater,
    ).run(ctx.obj["render_output_once"])


@performance.command(
    "topics", short_help="Show information about the performance topic types."
)
@click.pass_context
def topics(ctx):
    """Show information about the performance topic types."""
    console = Console(
        color_system=None if ctx.obj["color"] == "none" else ctx.obj["color"]
    )

    # KS
    console.rule("KS, Keyed Sequence (default)")
    console.print(
        Syntax(
            """@final
struct KeyedSeq
{
  unsigned long seq;         // sequence number
  @key unsigned long keyval; // key
  sequence<octet> baggage;   // variable-sized content
};""",
            "omg-idl",
        )
    )

    # K32
    console.rule("K32, Keyed 32 bytes")
    console.print(
        Syntax(
            """@final
struct Keyed32
{
  unsigned long seq;         // sequence number
  @key unsigned long keyval; // key
  octet baggage[24];         // content
};""",
            "omg-idl",
        )
    )

    # K256
    console.rule("K256, Keyed 256 bytes")
    console.print(
        Syntax(
            """@final
struct Keyed256
{
  unsigned long seq;         // sequence number
  @key unsigned long keyval; // key
  octet baggage[248];        // content
};""",
            "omg-idl",
        )
    )

    # OU
    console.rule("OU, One ULong")
    console.print(
        Syntax(
            """@final
struct OneULong
{
  unsigned long seq;         // sequence number
};""",
            "omg-idl",
        )
    )

    # UK16
    console.rule("UK16, Unkeyed 16 bytes")
    console.print(
        Syntax(
            """@final
struct Unkeyed16
{
  unsigned long seq; // sequence number
  octet baggage[12]; // content
};""",
            "omg-idl",
        )
    )

    # UK1024
    console.rule("UK1024, Unkeyed 1024 bytes")
    console.print(
        Syntax(
            """@final
struct Unkeyed1024
{
  unsigned long seq;   // sequence number
  octet baggage[1020]; // content
};""",
            "omg-idl",
        )
    )

    # S16
    console.rule("S16, Keyed, 16 octets")
    console.print(
        Syntax(
            """@final
struct Struct16 {
  octet struct0;
  octet struct1;
  octet struct2;
  octet struct3;
  octet struct4;
  octet struct5;
  octet struct6;
  octet struct7;
  octet struct8;
  octet struct9;
  octet structa;
  octet structb;
  octet structc;
  octet structd;
  octet structe;
  octet structf;
  long long junk;
  unsigned long seq;
  @key unsigned long keyval;
};""",
            "omg-idl",
        )
    )

    # S256
    console.rule("S256, Keyed, 16 Struct16's")
    console.print(
        Syntax(
            """@final
struct Struct256 {
  Struct16 struct160;
  Struct16 struct161;
  Struct16 struct162;
  Struct16 struct163;
  Struct16 struct164;
  Struct16 struct165;
  Struct16 struct166;
  Struct16 struct167;
  Struct16 struct168;
  Struct16 struct169;
  Struct16 struct16a;
  Struct16 struct16b;
  Struct16 struct16c;
  Struct16 struct16d;
  Struct16 struct16e;
  Struct16 struct16f;
  long long junk;
  unsigned long seq;
  @key unsigned long keyval;
};""",
            "omg-idl",
        )
    )

    # S4k
    console.rule("S4k, Keyed, 16 Struct256's")
    console.print(
        Syntax(
            """@final
struct Struct4k {
  Struct256 struct2560;
  Struct256 struct2561;
  Struct256 struct2562;
  Struct256 struct2563;
  Struct256 struct2564;
  Struct256 struct2565;
  Struct256 struct2566;
  Struct256 struct2567;
  Struct256 struct2568;
  Struct256 struct2569;
  Struct256 struct256a;
  Struct256 struct256b;
  Struct256 struct256c;
  Struct256 struct256d;
  Struct256 struct256e;
  Struct256 struct256f;
  long long junk;
  unsigned long seq;
  @key unsigned long keyval;
};""",
            "omg-idl",
        )
    )

    # S32k
    console.rule("S32k, Keyed, 8 Struct4k's")
    console.print(
        Syntax(
            """@final
struct Struct32k {
  Struct4k struct4k0;
  Struct4k struct4k1;
  Struct4k struct4k2;
  Struct4k struct4k3;
  Struct4k struct4k4;
  Struct4k struct4k5;
  Struct4k struct4k6;
  Struct4k struct4k7;
  long long junk;
  unsigned long seq;
  @key unsigned long keyval;
};""",
            "omg-idl",
        )
    )

    return None



"""
 * Copyright(c) 2021 to 2022 ZettaScale Technology and others
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License v. 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0, or the Eclipse Distribution License
 * v. 1.0 which is available at
 * http://www.eclipse.org/org/documents/edl-v10.php.
 *
 * SPDX-License-Identifier: EPL-2.0 OR BSD-3-Clause
"""

from dataclasses import fields
from typing import Sequence, List, Union

from cyclonedds.qos import BasePolicy, Qos, Policy
from cyclonedds.util import duration


dds_infinity = 9223372036854775807


class QosParser:

    # Policy mapping (lowercase) name -> class or instance
    policies = {k.lower(): v for k, v in Qos._policy_mapper.items()}

    def __init__(self, data: List[str]) -> None:
        self.pos = 0      # parser position in data
        self.data = data  # data to parse

    @staticmethod
    def prepare_arguments(arguments: List[str]):
        # Allow user to use : and , to freely visually split arguments
        # Parsable arguments are all split along all whitespace
        ret = []
        for arg in arguments:
            arg = arg.replace(":", "").replace(",", "")
            ret += arg.split()
        return ret

    @staticmethod
    def parse(arguments: List[str]) -> Qos:
        # Main method: qos = QosParser.parse(arguments)
        arguments = QosParser.prepare_arguments(arguments)
        parser = QosParser(arguments)
        return Qos(*parser.parse_list_of_policies())

    def parse_list_of_policies(self) -> List[BasePolicy]:
        # Build policies until end of string is reached
        ret = []
        while not self.at_end():
            ret.append(self.parse_policy())

        return ret

    def parse_policy(self):
        # Construct a policy. First argument should always resolve to a policy name
        name = self.string().lower()
        name = name if name.startswith("policy.") else f"policy.{name}"

        if name not in self.policies:
            raise Exception(f"No such policy {name}")

        policy = self.policies[name]
        argument_types = [f.type for f in fields(policy)]

        if argument_types:
            # There is a non-zero amount of typed arguments to parse
            arguments = [self.parse_argument_of_type(_type) for _type in argument_types]
            return policy(*arguments)
        else:
            # There are no arguments to parse so the policy is already complete
            return policy

    def parse_argument_of_type(self, _type):
        # Typed dispatch to simple parsers

        if _type == str:
            return self.string()
        elif _type == int:
            return self.integer()
        elif _type == float:
            return self.floating()
        elif _type == bool:
            return self.boolean()
        elif _type == Sequence[str]:
            return self.string_list()
        elif _type == bytes:
            return self.binary_data()
        elif _type == Union['Policy.History.KeepAll', 'Policy.History.KeepLast']:
            # This is a special case for DurabilityService which contains a History Policy
            ret = self.parse_policy()
            if ret.__scope__ != "History":
                raise Exception("DurabilityService takes a History policy")
            return ret

        raise Exception(f"Cannot parse type {_type}, this is a bug, please report it.")

    def pop(self) -> str:
        # Safely return the next item in the parse list and increment the position
        if self.pos >= len(self.data):
            raise Exception("Unexpected end of arguments")

        ret = self.data[self.pos]
        self.pos += 1
        return ret

    def peek(self) -> str:
        # Safely peek the next item in the parse list
        if self.pos >= len(self.data):
            raise Exception("Unexpected end of arguments")

        return self.data[self.pos]

    def at_end(self) -> bool:
        # Are we done parsing?
        return self.pos >= len(self.data)

    def string(self):
        # Strings are always valid, just return the next item
        return self.pop()

    def integer(self):
        # Integers can represent: a duration or an amount
        data = self.pop().lower()

        if '=' in data:
            # Allow writing durations like "seconds=10;minutes=12"
            duration_expression = {k: float(v) for k, v in dict(value.split("=") for value in data.split(';')).items()}
            data = duration(**duration_expression)
        elif data in ["infinity", "inf"]:
            data = dds_infinity
        else:
            data = int(data)

        return data

    def floating(self):
        # Floating points are never durations so don't parse it as that.
        return float(self.pop())

    def boolean(self):
        # true or false, support some other ways of saying that
        data = self.pop().lower()

        if data in ["true", "1", "on", "yes"]:
            return True
        elif data in ["false", "0", "off", "no"]:
            return False

        raise Exception(f"Invalid boolean {data}")

    def string_list(self):
        # String lists are precarious, since it is unclear when they end.
        # We will stop when we recognize another Policy or there is nothing left to parse

        ret = []
        while not self.at_end():
            name = self.peek().lower()
            name = name if name.startswith("policy.") else f"policy.{name}"

            if name in self.policies:
                break

            ret.append(self.string())

        return ret

    def binary_data(self):
        # Support Userdata, Groupdata or Topicdata as base64 encoded string.
        data = self.string()
        return data.encode()



import re
from rich.layout import Layout
from rich.text import Text

from .app import Header, CPUGraph, ScrollGraph, PeerPanel
from .barchart import RichChart


def make_sub_layout(cmd) -> Layout:
    layout = Layout(name="ddsperf_app")

    layout.split(
        Layout(name="header", size=3),
        Layout(name="main1", ratio=1),
        Layout(name="main2", ratio=1),
    )
    layout["main1"].split_row(
        Layout(name="samplerate", ratio=3),
        Layout(name="hist", ratio=2),
        Layout(name="peers", ratio=1),
    )
    layout["main2"].split_row(
        Layout(name="cpu", ratio=2),
        Layout(name="cpu_legend", ratio=1),
        Layout(name="mini1", ratio=1),
        Layout(name="mini2", ratio=1),
        Layout(name="mini3", ratio=1),
    )
    layout["mini1"].split_column(Layout(name="maxrss"), Layout(name="discarded"))
    layout["mini2"].split_column(Layout(name="rexmit"), Layout(name="trexmit"))
    layout["mini3"].split_column(Layout(name="tthrottle"), Layout(name="nthrottle"))

    layout["header"].update(Header(" ".join(cmd)))
    layout["cpu"].update(Text(""))
    layout["cpu_legend"].update(Text(""))
    layout["samplerate"].update(Text(""))
    layout["hist"].update(Text(""))
    layout["peers"].update(Text(""))
    layout["maxrss"].update(Text(""))
    layout["discarded"].update(Text(""))
    layout["rexmit"].update(Text(""))
    layout["trexmit"].update(Text(""))
    layout["tthrottle"].update(Text(""))
    layout["nthrottle"].update(Text(""))
    return layout


def make_sub_updater():
    data_re = re.compile(
        r"\[\d*\] [\d\.]+  size \d+ total \d+ lost \d+ delta \d+ lost \d+ rate ([\d\.]+) kS/s ([\d\.]+)"
    )
    csw_cpu_re = re.compile(r"vcsw:(\d+) ivcsw:(\d+)((?: \w+:\d+%\+\d+%)*)")
    cpu_singlet_re = re.compile(r"(\w+):(\d+)%\+(\d+)%")
    peer_re = re.compile(r"\[\d+\] participant ([^:]+):(\d+): (.*)$")
    rss_re = re.compile(r"rss:([\d\.]+)(kb|MB)")
    discarded_re = re.compile(r"discarded ([\d\.]+)")
    rexmit_re = re.compile(r" rexmit ([\d\.]+)")
    trexmit_re = re.compile(r" Trexmit ([\d\.]+)")
    tthrottle_re = re.compile(r" Tthrottle ([\d\.]+)")
    nthrottle_re = re.compile(r" Nthrottle ([\d\.]+)")

    y = []
    graphs = {}
    graphs["cpu"] = CPUGraph(10)
    graphs["peers"] = PeerPanel()
    graphs["samplerate"] = ScrollGraph("Samplerate(S/s)", 20, "bright_red")
    graphs["maxrss"] = ScrollGraph("MaxRSS", 10, "bright_cyan")
    graphs["discarded"] = ScrollGraph("Discarded msg", 10, "bright_green", delta=True)
    graphs["rexmit"] = ScrollGraph("Rexmit", 10, "bright_green", delta=True)
    graphs["trexmit"] = ScrollGraph("TRexmit", 10, "bright_green")
    graphs["tthrottle"] = ScrollGraph("Tthrottle", 10, "bright_green")
    graphs["nthrottle"] = ScrollGraph("Nthrottle", 10, "bright_green", delta=True)
    graphs["hist"] = RichChart(title="Samplerate histogram")

    def updater(line):
        updated = set()

        m = data_re.match(line)
        if m:
            updated.add("samplerate")
            updated.add("hist")
            y.append(float(m.group(1)) * 1000.0)
            if len(y) > 10000:
                y.pop(0)
            graphs["samplerate"].add_point(float(m.group(1)) * 1000.0)
            graphs["hist"].hist(y, "bright_red", index=0)

        m2 = csw_cpu_re.search(line)
        if m2:
            updated.add("cpu")
            graphs["cpu"].next_data(int(m2.group(1)), int(m2.group(2)))
            for m3 in cpu_singlet_re.finditer(m2.group(3)):
                name, user, system = m3.group(1), int(m3.group(2)), int(m3.group(3))
                graphs["cpu"].report_stats(name, user, system)

        rss = rss_re.search(line)
        if rss:
            updated.add("maxrss")
            graphs["maxrss"].add_point(
                float(rss.group(1)) * (1000000 if rss.group(2) == "MB" else 1000)
            )

        discarded_m = discarded_re.search(line)
        if discarded_m:
            updated.add("discarded")
            graphs["discarded"].add_point(int(discarded_m.group(1)))

        rexmit_m = rexmit_re.search(line)
        if rexmit_m:
            updated.add("rexmit")
            graphs["rexmit"].add_point(int(rexmit_m.group(1)))

        trexmit_m = trexmit_re.search(line)
        if trexmit_m:
            updated.add("trexmit")
            graphs["trexmit"].add_point(int(trexmit_m.group(1)))

        tthrottle_m = tthrottle_re.search(line)
        if tthrottle_m:
            updated.add("tthrottle")
            graphs["tthrottle"].add_point(int(tthrottle_m.group(1)))

        nthrottle_m = nthrottle_re.search(line)
        if nthrottle_m:
            updated.add("nthrottle")
            graphs["nthrottle"].add_point(int(nthrottle_m.group(1)))

        peer_m = peer_re.match(line)
        if peer_m:
            updated.add("peers")
            host, pid, action = peer_m.group(1), peer_m.group(2), peer_m.group(3)
            if action == "new (self)":
                graphs["peers"].set_own(host, pid)
            elif action == "new":
                graphs["peers"].add_peer(host, pid)
            elif action == "gone":
                graphs["peers"].remove_peer(host, pid)

        return updated

    return updater, graphs



import pytest

from cyclonedds.core import DDSException
from cyclonedds.domain import DomainParticipant
from cyclonedds.pub import Publisher
from cyclonedds.util import isgoodentity


def test_initialize_publisher():
    dp = DomainParticipant(0)
    pub = Publisher(dp)

    assert isgoodentity(pub)


def test_suspension():
    dp = DomainParticipant(0)
    pub = Publisher(dp)

    with pytest.raises(DDSException) as exc:
        pub.suspend()
    assert exc.value.code == DDSException.DDS_RETCODE_UNSUPPORTED

    with pytest.raises(DDSException) as exc:
        pub.resume()
    assert exc.value.code == DDSException.DDS_RETCODE_UNSUPPORTED



import asyncio
import re
import asyncio.subprocess as asp
from typing import Optional

from rich.console import Console
from rich.layout import Layout
from rich.panel import Panel
from rich.text import Text
from rich.live import Live

from .barchart import RichChart


class Header:
    """Display header with clock."""

    def __init__(self, title) -> None:
        self.title = title

    def __rich__(self) -> Panel:
        return Panel(self.title, style="bold bright_blue")


class CPUGraph:
    _colors = ["green", "blue", "magenta", "cyan", "white", "red"]

    def __init__(self, history_size, **kwargs) -> None:
        self.vcsw = 0
        self.ivcsw = 0
        self._thread_stats = {}
        self._color_index = 0
        self._history_size = history_size
        self._chart_kwargs = kwargs

    def update_chart_vars(self, **kwargs):
        self._chart_kwargs.update(kwargs)

    def report_stats(self, name, user_value, system_value):
        if name not in self._thread_stats:
            self._init_thread(name)
        self._thread_stats[name][1][self._history_size - 1] = user_value
        self._thread_stats[name][2][self._history_size - 1] = system_value

    def next_data(self, vcsw, ivcsw):
        self.vcsw = vcsw
        self.ivcsw = ivcsw
        for name in self._thread_stats.keys():
            for i in range(self._history_size - 1):
                self._thread_stats[name][1][i] = self._thread_stats[name][1][i + 1]
                self._thread_stats[name][2][i] = self._thread_stats[name][2][i + 1]
            self._thread_stats[name][1][self._history_size - 1] = 0.0
            self._thread_stats[name][2][self._history_size - 1] = 0.0

    def _init_thread(self, name):
        self._thread_stats[name] = [
            [],
            [0.0] * self._history_size,
            [0.0] * self._history_size,
            self._colors[self._color_index]
            if self._color_index < len(self._colors)
            else "blue",
        ]
        self._color_index += 1
        for i, value in enumerate(self._thread_stats.values()):
            value[0] = [
                j + i / 2 / float(self._color_index) for j in range(self._history_size)
            ]

    def draw(self):
        chart = RichChart(title="[bold]CPU(%thread)[/]", **self._chart_kwargs)
        for values in self._thread_stats.values():
            chart.plot(values[0], values[1], "bright_" + values[3])
            chart.plot(values[0], values[2], "dark_" + values[3])
        chart.ylim((0, 100))
        return chart

    def legend(self):
        return Text.from_markup(
            "\n"
            + "\n".join(
                f"[bold][bright_{v[3]}]{k} user[/]/[dark_{v[3]}]{k} system[/][/]"
                for k, v in self._thread_stats.items()
            )
            + f"\n\n[bold cornflower_blue]Context switches[/]\n[bold]Voluntary: {self.vcsw}\nInvoluntary: {self.ivcsw}"
        )


class ScrollGraph:
    def __init__(
        self, title, history_size, color, delta: bool = False, **kwargs
    ) -> None:
        self._title = title
        self._history_size = history_size
        self._color = color
        self._data = [0.0] * history_size
        self._limits = None
        self._chart_kwargs = kwargs
        self._delta = delta
        self._time = [i for i in range(history_size)]

    def add_point(self, value):
        if self._delta:
            value = value - self._data[self._history_size - 2]

        if not self._limits:
            self._limits = [value, value]
        else:
            self._limits[0] = min(self._limits[0], value)
            self._limits[1] = max(self._limits[1], value)

        for i in range(self._history_size - 1):
            self._data[i] = self._data[i + 1]
        self._data[self._history_size - 1] = value

    def draw(self):
        chart = RichChart(
            title=f"[bold {self._color}]{self._title}[/]", **self._chart_kwargs
        )
        chart.plot(self._time, self._data, self._color)
        if self._limits:
            chart.ylim(self._limits)
        return chart

    def update_chart_vars(self, **kwargs):
        self._chart_kwargs.update(kwargs)


class MultiScrollGraph:
    _default_colors = [
        "bright_green",
        "bright_blue",
        "bright_magenta",
        "bright_cyan",
        "bright_white",
        "bright_red",
    ]

    def __init__(self, title, history_size, delta: bool = False, **kwargs) -> None:
        self._title = title
        self._history_size = history_size
        self._colors = []
        self._data = []
        self._limits = None
        self._chart_kwargs = kwargs
        self._delta = delta
        self._time = [i for i in range(history_size)]

    def add_new_line(self, color: Optional[str] = None):
        color = color or self._default_colors[len(self._data)]
        self._data.append([0.0] * self._history_size)
        self._colors.append(color)
        return len(self._colors) - 1

    def add_point(self, index, value):
        if self._delta:
            value = value - self._data[index][self._history_size - 2]

        if not self._limits:
            self._limits = [value, value]
        else:
            self._limits[0] = min(self._limits[0], value)
            self._limits[1] = max(self._limits[1], value)

        for i in range(self._history_size - 1):
            self._data[index][i] = self._data[index][i + 1]
        self._data[index][self._history_size - 1] = value

    def draw(self):
        chart = RichChart(title=self._title, **self._chart_kwargs)
        for i in range(len(self._data)):
            chart.plot(self._time, self._data[i], self._colors[i])
        if self._limits:
            chart.ylim(self._limits)
        return chart

    def update_chart_vars(self, **kwargs):
        self._chart_kwargs.update(kwargs)


class PeerPanel:
    _colors = [
        "bright_green",
        "bright_blue",
        "bright_magenta",
        "bright_cyan",
        "bright_white",
        "bright_red",
    ]

    def __init__(self, coloured: bool = False):
        self.my_peer = None
        self.peers = []
        self.historic_peers = []
        self.colored = coloured
        self.colors = {}

    def add_peer(self, host, pid):
        if (host, pid) in self.historic_peers:
            self.historic_peers.remove((host, pid))
        self.peers.append((host, pid))

    def remove_peer(self, host, pid):
        peer = (host, pid)
        self.peers.remove(peer)
        if self.colored and peer in self.colors:
            del self.colors[peer]
        self.historic_peers.append(peer)

    def set_own(self, host, pid):
        self.my_peer = (host, pid)

    def get_peer_color(self, host, pid):
        peer = (host, pid)
        if not self.colored:
            return "bright_cyan"

        if peer in self.colors:
            return self.colors[peer]

        used = set(self.colors.values())
        for c in self._colors:
            if c not in used:
                self.colors[peer] = c
                return c

        self.colors[peer] = "bright_yellow"
        return "bright_yellow"

    def draw(self):
        if not self.my_peer:
            return Panel("waiting for data")

        txt = f"[bold]╶── Self ──╴\n[{self.get_peer_color(*self.my_peer)}]{self.my_peer[0]}[/]:{self.my_peer[1]}[/]\n\n[bold]╶── Others ──╴[/]\n"

        txt += "\n".join(
            f"[{self.get_peer_color(*e)}]{e[0]}[/]:{e[1]}" for e in self.peers
        )

        if self.historic_peers:
            txt += "\n\n[bold]╶── Gone ──╴[/]\n"
            txt += "\n".join(f"[magenta]{e[0]}[/]:{e[1]}" for e in self.historic_peers)

        return Panel(txt, title="Peers")

    def update_chart_vars(self, **kwargs):
        pass


class DDSPerfApp:
    def __init__(self, console, cmd, layout_maker, update_maker) -> None:
        self.updater, self.graphs = update_maker()
        self.layout = layout_maker(cmd)
        self.cmd = cmd
        self.ph = 0
        self.console = console
        self.to_update = set()
        self.proc = None

    def add(self, a_set):
        self.to_update.update(a_set)

    def height_adjust(self):
        if self.ph == self.console.height:
            return

        self.ph = self.console.height
        render = self.layout.render(self.console, self.console.options)
        for graph in self.graphs.keys():
            if hasattr(self.graphs[graph], "update_chart_vars"):
                self.graphs[graph].update_chart_vars(
                    height=render[self.layout.get(graph)].region.height
                )
            self.add({graph})

    def refresh(self):
        for graph in self.to_update:
            if graph == "cpu":
                self.layout["cpu_legend"].update(self.graphs["cpu"].legend())
            self.layout[graph].update(
                self.graphs[graph].draw()
                if hasattr(self.graphs[graph], "draw")
                else self.graphs[graph]
            )
        self.to_update = set()
        self.height_adjust()

    async def _iter_output(self):
        while True:
            line = (await self.proc.stdout.readline()).decode().strip()
            self.add(self.updater(line))

    async def _cancel_on_quit(self, task):
        await self.proc.wait()
        task.cancel()

    async def process_ddsperf(self):
        self.proc = await asp.create_subprocess_shell(
            cmd=" ".join(self.cmd), stdout=asp.PIPE
        )
        t1 = asyncio.create_task(self._iter_output())
        t2 = asyncio.create_task(self._cancel_on_quit(t1))
        await asyncio.wait((t1, t2), return_when=asyncio.FIRST_COMPLETED)

    async def screen_refresh(self, live):
        while True:
            await asyncio.sleep(0.5)
            self.refresh()
            live.refresh()

    def run(self, render_once_on_exit):
        try:
            asyncio.run(self.run_inner(render_once_on_exit))
        except KeyboardInterrupt:
            pass

    async def run_inner(self, render_once_on_exit):
        if render_once_on_exit:
            await self.process_ddsperf()
            self.height_adjust()
            self.refresh()
            self.console.print(self.layout)
            return

        self.height_adjust()
        with Live(
            self.layout, console=self.console, auto_refresh=False, screen=True
        ) as live:
            try:
                done, pending = await asyncio.wait(
                    (
                        self.screen_refresh(live),
                        self.process_ddsperf(),
                    ),
                    return_when=asyncio.FIRST_COMPLETED,
                )
                for p in pending:
                    p.cancel()
            except KeyboardInterrupt:
                self.proc.terminate()
                await self.proc.wait()



"""
 * Copyright(c) 2021 to 2022 ZettaScale Technology and others
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License v. 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0, or the Eclipse Distribution License
 * v. 1.0 which is available at
 * http://www.eclipse.org/org/documents/edl-v10.php.
 *
 * SPDX-License-Identifier: EPL-2.0 OR BSD-3-Clause
"""

import platform
from wheel.bdist_wheel import bdist_wheel as _bdist_wheel
from cyclone_search import find_cyclonedds
from pathlib import Path
import shutil
import os


class bdist_wheel(_bdist_wheel):
    def initialize_options(self):
        self.standalone = os.environ.get("STANDALONE_WHEELS") == "1"
        super().initialize_options()

    def finalize_options(self):
        if self.standalone:
            self.distribution.entry_points["console_scripts"].append("idlc=cyclonedds.tools.wheel_idlc:command")
        super().finalize_options()

    def run(self):
        if self.standalone:
            cyclone = find_cyclonedds()

            newlibdir = Path(self.bdist_dir) / 'cyclonedds' / '.libs'

            os.makedirs(newlibdir, exist_ok=True)
            (Path(self.bdist_dir) / 'cyclonedds' / '__library__.py').write_text(
                (Path(__file__).parent / "wheel_library.py").read_text()
            )

            if not platform.system() == "Darwin":
                shutil.copy(cyclone.ddsc_library, newlibdir / cyclone.ddsc_library.name)

            if cyclone.idlc_executable and cyclone.idlc_library:
                shutil.copy(cyclone.idlc_executable, newlibdir / cyclone.idlc_executable.name)

        super().run()



# will be written to __library__.py
from pathlib import Path
import platform

in_wheel = True

dir = Path(__file__).parent.resolve()

if (dir / ".." / "cyclonedds_nightly.libs").exists():
    if platform.system() == "Windows":
        library_path = list((dir / ".." / "cyclonedds_nightly.libs").glob("ddsc*"))[0]
    else:
        library_path = list((dir /  ".." / "cyclonedds_nightly.libs").glob("libddsc*"))[0]

else:
    if platform.system() == "Darwin":
        library_path = list((dir / ".dylibs").glob("libddsc*"))[0]
    elif platform.system() == "Windows":
        library_path = list((dir / ".." / "cyclonedds.libs").glob("ddsc*"))[0]
    else:
        library_path = list((dir / ".." / "cyclonedds.libs").glob("libddsc*"))[0]



import pytest
import itertools
from cyclonedds.qos import Policy, Qos, _CQos, TopicQos


some_qosses = [
    Qos(Policy.Reliability.BestEffort),
    Qos(Policy.Reliability.Reliable(22)),
    Qos(Policy.Durability.Volatile),
    Qos(Policy.Durability.TransientLocal),
    Qos(Policy.Durability.Transient),
    Qos(Policy.Durability.Persistent),
    Qos(Policy.History.KeepAll),
    Qos(Policy.History.KeepLast(10)),
    Qos(Policy.ResourceLimits(3, 4, 5)),
    Qos(Policy.PresentationAccessScope.Instance(False, True)),
    Qos(Policy.PresentationAccessScope.Topic(True, True)),
    Qos(Policy.PresentationAccessScope.Group(False, False)),
    Qos(Policy.Lifespan(12001)),
    Qos(Policy.Deadline(2129981)),
    Qos(Policy.LatencyBudget(1337)),
    Qos(Policy.Ownership.Shared),
    Qos(Policy.Ownership.Exclusive),
    Qos(Policy.OwnershipStrength(8)),
    Qos(Policy.Liveliness.Automatic(898989)),
    Qos(Policy.Liveliness.ManualByParticipant(898989)),
    Qos(Policy.Liveliness.ManualByTopic(898989)),
    Qos(Policy.TimeBasedFilter(999900999)),
    Qos(Policy.Partition(["a", "b", "8isdfijsdifij3e8"])),
    Qos(Policy.TransportPriority(9)),
    Qos(Policy.DestinationOrder.ByReceptionTimestamp),
    Qos(Policy.DestinationOrder.BySourceTimestamp),
    Qos(Policy.WriterDataLifecycle(False)),
    Qos(Policy.ReaderDataLifecycle(7, 9)),
    Qos(Policy.DurabilityService(12, Policy.History.KeepAll, 99, 88, 77)),
    Qos(Policy.DurabilityService(112, Policy.History.KeepLast(66), 199, 188, 177)),
    Qos(Policy.IgnoreLocal.Nothing),
    Qos(Policy.IgnoreLocal.Participant),
    Qos(Policy.IgnoreLocal.Process),
    Qos(Policy.Userdata(b"1298129891lsakdjflksadjflas")),
    Qos(Policy.Groupdata(b"\0ksdlfkjsldkfj")),
    Qos(Policy.Topicdata(b"\n\nrrlskdjflsdj")),
    Qos(Policy.Userdata(b"1298129891lsakdjflksadjflas"),
        Policy.Groupdata(b"\0ksdlfkjsldkfj"),
        Policy.Topicdata(b"\n\nrrlskdjflsdj")),
    Qos(Policy.Property("a", "bs")),
    Qos(Policy.BinaryProperty("a", b"bs")),
    Qos(Policy.Property("a", "bs"), Policy.Property("b", "cs")),
    Qos(Policy.TypeConsistency.AllowTypeCoercion(True, False, True, False, True)),
    Qos(Policy.TypeConsistency.AllowTypeCoercion(False, False, False, True, True)),
    Qos(Policy.TypeConsistency.AllowTypeCoercion(True, True, False, False, False)),
    Qos(Policy.TypeConsistency.DisallowTypeCoercion(True)),
    Qos(Policy.TypeConsistency.DisallowTypeCoercion(False)),
    Qos(Policy.DataRepresentation(use_cdrv0_representation=True)),
    Qos(Policy.DataRepresentation(use_xcdrv2_representation=True)),
    Qos(Policy.DataRepresentation(use_cdrv0_representation=True, use_xcdrv2_representation=True)),
    Qos(Policy.EntityName("dihydrogenmonoxide"))
]

qos_pairs = list(itertools.combinations(some_qosses, 2))

def to_c_and_back(qos):
    cqos = _CQos.qos_to_cqos(qos)
    nqos = _CQos.cqos_to_qos(cqos)
    _CQos.cqos_destroy(cqos)
    return nqos


def test_qos_ops():
    for qos in some_qosses:
        assert qos == to_c_and_back(qos)
        assert qos == Qos.fromdict(qos.asdict())
        for policy in qos:
            assert policy in qos
        repr(qos)
        qos.domain_participant()
        qos.subscriber()
        qos.publisher()
        qos.topic()
        qos.datawriter()
        qos.datareader()
        assert qos - qos == Qos()
        assert qos + qos == qos


def test_qos_inequality():
    for qos1, qos2 in qos_pairs:
        assert qos1 != qos2


def test_qos_lookup():
    qos = Qos(Policy.Durability.Volatile)
    assert qos[Policy.Durability] == Policy.Durability.Volatile
    qos = Qos(Policy.History.KeepLast(20))
    assert qos[Policy.History] == Policy.History.KeepLast(20)
    assert qos[Policy.History.KeepLast] == Policy.History.KeepLast(20)


def test_qos_inheritance():
    qos = Qos(Policy.Durability.Volatile)
    qos2 = Qos(Policy.Deadline(10), base=qos)
    qos3 = Qos(Policy.Durability.Transient, base=qos)
    assert qos2 == Qos(Policy.Durability.Volatile, Policy.Deadline(10)) == Qos(Policy.Deadline(10), Policy.Durability.Volatile)
    assert qos3 == Qos(Policy.Durability.Transient)
    assert qos3 != Qos()
    assert qos3 != Qos(Policy.Durability.Volatile)


def test_qos_raise_wrong_usage():
    with pytest.raises(NotImplementedError):
        Policy()

    with pytest.raises(NotImplementedError):
        Policy.Durability()

    with pytest.raises(TypeError):
        Policy.Durability.Persistent()

    with pytest.raises(TypeError):
        Qos(1)

    with pytest.raises(ValueError):
        Qos(
            Policy.Durability.Persistent,
            Policy.Durability.Volatile
        )

    with pytest.raises(ValueError):
        Qos(
            Policy.Property("a", "b"),
            Policy.Property("a", "c")
        )

    with pytest.raises(ValueError):
        Qos.fromdict({"Durability": {}})




from threading import Thread
from code import InteractiveConsole
import rich_click as click
import json
from rich import print
from rich.syntax import Syntax
from rich.prompt import Prompt
from rich.console import Console
from cyclonedds import domain, pub, topic as cyclone_topic
from cyclonedds.qos import Qos

try:
    import readline
except ImportError:
    pass

from .utils import (
    TimeDeltaParamType,
    LiveData,
    background_progress_viewer,
    background_printer,
)
from .discovery.main import type_discovery
from .data import subscribe as data_subscribe
from .common import select_type, select_qos


@click.command(short_help="Publish to an arbitrary topic")
@click.argument("topic")
@click.option(
    "-i", "--id", "--domain-id", type=int, help="DDS Domain to inspect."
)
@click.option(
    "-r",
    "--runtime",
    type=TimeDeltaParamType(),
    default="1s",
    help="Duration of discovery scan.",
)
@click.option(
    "--suppress-progress-bar",
    type=bool,
    is_flag=True,
    help="Suppress the output of the progress bar",
)
@click.option(
    "--color",
    type=click.Choice(["auto", "standard", "256", "truecolor", "windows", "none"]),
    default="auto",
    help="""Force the command to output with/without terminal colors. By default output colours if the terminal supports it."
See the [underline blue][link=https://rich.readthedocs.io/en/stable/console.html#color-systems]Rich documentation[/link][/] for more info on what the options mean.""",
)
@click.option(
    "--qos",
    type=click.Choice(["scan", "scan-random", "dds-default", "json"]),
    default="scan",
    help="""Method to determine the QoS settings of the writer. With "scan" the network is scanned for existing QoS used.
"scan-random" functions the same way but does not prompt for input, but simply picks a random QoS in case of conflicts.
With "dds-default" the default QoS from the DDS specification is used and with  "json" your first line of input should be a
json string with the QoS settings, defined by the output of `cyclonedds.qos.Qos.asdict()`.""",
)
@click.option(
    "--type",
    type=click.Choice(["scan", "scan-random"]),
    default="scan",
    help="""Method to determine the datatype of the writer. With "scan" the network is scanned for existing types using XTypes.
"scan-random" functions the same way but does not prompt for input, but simply picks a random datatype in case of conflicts.""",
)
def publish(topic, id, runtime, suppress_progress_bar, color, qos, type):
    """Publish to an arbitrary topic"""

    if qos == "json":
        try:
            qos_endpoint = Qos.fromdict(json.loads(input()))
            qos_topic = qos_endpoint
        except:
            return 1

    console = Console(color_system=None if color == "none" else color)
    live = LiveData(console)

    thread = Thread(target=type_discovery, args=(live, id, runtime, topic))
    thread.start()

    background_progress_viewer(runtime, live, suppress_progress_bar)

    thread.join()

    if qos in ["scan", "scan-random"]:
        qos_topic, qos_endpoint = select_qos(
            console, live.result, True, qos == "scan-random"
        )
    elif qos == "dds-default":
        qos_topic, qos_endpoint = Qos(), Qos()

    discovered_type = select_type(console, live.result, type == "scan-random")

    if not discovered_type:
        return

    console.print("Your datatypes are:")

    console.print(Syntax(discovered_type.code, "omg-idl"))
    console.print(
        "Available in your python REPL: [bold green]participant, topic, writer, [bold magenta]"
        + ", ".join(discovered_type.nested_dtypes.keys())
    )
    console.print("[bold green] Publishing, run exit() to quit")

    if id is None:
        dp = domain.DomainParticipant()
    else:
        dp = domain.DomainParticipant(id)
    tp = cyclone_topic.Topic(dp, topic, discovered_type.dtype, qos=qos_topic)
    dw = pub.DataWriter(dp, tp, qos=qos_endpoint)

    vars = {"participant": dp, "topic": tp, "writer": dw}
    vars.update(discovered_type.nested_dtypes)

    repl = InteractiveConsole(locals=vars)
    repl.push("from rich import pretty; pretty.install()")
    repl.interact(banner="")



"""
 * Copyright(c) 2021 to 2022 ZettaScale Technology and others
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License v. 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0, or the Eclipse Distribution License
 * v. 1.0 which is available at
 * http://www.eclipse.org/org/documents/edl-v10.php.
 *
 * SPDX-License-Identifier: EPL-2.0 OR BSD-3-Clause
"""

import sys
import struct

from dataclasses import dataclass, field
from enum import IntEnum, Enum, auto
from typing import Any, List, Optional, Tuple


class CdrKeyVMOpType(IntEnum):
    Done = 0
    StreamStatic = 1
    Stream2ByteSize = 2
    Stream4ByteSize = 3
    ByteSwap = 4
    RepeatStatic = 5
    Repeat2ByteSize = 6
    Repeat4ByteSize = 7
    EndRepeat = 8
    Union1Byte = 9
    Union2Byte = 10
    Union4Byte = 11
    Union8Byte = 12
    Jump = 13
    Optional = 14
    MemberSelect = 15
    MemberSelectEnd = 16
    StructHeader = 17
    AppendableHeader = 18
    AppendableJumpToEnd = 19


@dataclass
class CdrKeyVmOp:
    type: CdrKeyVMOpType
    skip: bool
    size: int = 0
    value: int = 0
    align: int = 0


@dataclass
class CdrKeyVmNamedJumpOp(CdrKeyVmOp):
    jumpto: str = ""


class Endianness(Enum):
    Little = auto()
    Big = auto()

    @staticmethod
    def native() -> 'Endianness':
        return Endianness.Little if sys.byteorder == "little" else Endianness.Big

# Used to indicate the output of the serialization:
#     DataSample: serialize the sample
#     KeyDefinitionOrder: serialize key fields only, in definition order
#     KeyNormalized: serialize key fields in normalized order, which means that for mutable structs,
#         the members are sorted by their member id (used in the fuzzer tests for mutated types)
class SerializeKind(Enum):
    DataSample = auto()
    KeyDefinitionOrder = auto()
    KeyNormalized = auto()

class DeserializeKind(Enum):
    DataSample = auto()
    KeySample = auto()

class Buffer:
    def __init__(self, _bytes: Optional[bytes] = None, align_offset: int = 0, align_max: int = 8) -> None:
        self._bytes: bytearray = bytearray(_bytes) if _bytes else bytearray(512)
        self._pos: int = 0
        self._size: int = len(self._bytes)
        self._align_offset: int = align_offset
        self._align_max: int = align_max
        self.set_endianness(Endianness.native())

    def set_endianness(self, endianness: Endianness) -> None:
        self.endianness = endianness
        if self.endianness == Endianness.Little:
            self._endian = "<"
        else:
            self._endian = ">"

    def zero_out(self) -> None:
        # As per testing (https://stackoverflow.com/questions/19671145)
        # Quickest way to zero is to re-alloc..
        self._bytes = bytearray(self._size)

    def set_align_offset(self, offset: int) -> int:
        old = self._align_offset
        self._align_offset = offset
        return old

    def seek(self, pos: int) -> 'Buffer':
        self._pos = pos
        return self

    def tell(self) -> int:
        return self._pos

    def ensure_size(self, size: int) -> None:
        if self._pos + size > self._size:
            old_bytes = self._bytes
            old_size = self._size
            while self._pos + size > self._size:
                self._size *= 2
            self._bytes = bytearray(self._size)
            self._bytes[0:old_size] = old_bytes

    def align(self, alignment: int) -> 'Buffer':
        alignment = min(alignment, self._align_max)
        self._pos = ((self._pos - self._align_offset + alignment - 1) & ~(alignment - 1)) + self._align_offset
        return self

    def write(self, pack: str, size: int, value: Any) -> 'Buffer':
        self.ensure_size(size)
        struct.pack_into(self._endian + pack, self._bytes, self._pos, value)
        self._pos += size
        return self

    def write_bytes(self, _bytes: bytes) -> 'Buffer':
        length = len(_bytes)
        self.ensure_size(length)
        self._bytes[self._pos:self._pos + length] = _bytes
        self._pos += length
        return self

    def write_multi(self, pack: str, size: int, *values: Any) -> 'Buffer':
        self.ensure_size(size)
        struct.pack_into(self._endian + pack, self._bytes, self._pos, *values)
        self._pos += size
        return self

    def read_bytes(self, length: int) -> bytes:
        b = bytes(self._bytes[self._pos:self._pos + length])
        self._pos += length
        return b

    def read(self, pack: str, size: int) -> Any:
        v = struct.unpack_from(self._endian + pack, buffer=self._bytes, offset=self._pos)
        self._pos += size
        return v[0]

    def read_multi(self, pack: str, size: int) -> Tuple[Any, ...]:
        v = struct.unpack_from(self._endian + pack, buffer=self._bytes, offset=self._pos)
        self._pos += size
        return v

    def asbytes(self) -> bytes:
        return bytes(self._bytes[0:self._pos])


class KeyScanResult(Enum):
    FixedSize = 1
    BoundSize = 2
    PossiblyInfinite = 3


@dataclass
class KeyScanner:
    entries: List[List[Tuple[int, int]]] = field(default_factory=list)
    rtype: KeyScanResult = KeyScanResult.FixedSize

    @classmethod
    def infinity(cls):
        return cls(rtype=KeyScanResult.PossiblyInfinite)

    @classmethod
    def simple(cls, alignment, size):
        return cls(entries=[[(alignment, size)]])

    @classmethod
    def with_bound(cls, alignment, bound):
        return cls(entries=[[(alignment, bound)]], rtype=KeyScanResult.BoundSize)

    @property
    def size(self):
        if self.rtype != KeyScanResult.PossiblyInfinite:
            size = 0
            for possible_route in self.entries:
                ssize = 0
                for (alignment, subsize) in possible_route:
                    ssize = ((ssize + alignment - 1) & ~(alignment - 1))
                    ssize += subsize
                size = max(size, ssize)
            return size
        return 1_000_000_000

    def increase_by_bytes(self, alignment, size):
        if self.rtype != KeyScanResult.PossiblyInfinite:
            if not self.entries:
                self.entries = [(alignment, size)]
            else:
                for entry in self.entries:
                    entry.append((alignment, size))

    def increase_by_multiplied_subresult(self, subresult, times):
        if self.rtype == KeyScanResult.PossiblyInfinite:
            return

        if subresult.rtype == KeyScanResult.PossiblyInfinite:
            self.rtype = KeyScanResult.PossiblyInfinite
            self.entries.clear()
            return

        if subresult.entries and len(subresult.entries) == 1 and len(subresult.entries[0]) == 1:
            # special case to keep things managable
            entry = (subresult.entries[0][0][0], subresult.entries[0][0][1] * times)
            if not self.entries:
                self.entries = [[entry]]
            else:
                self.entries = [entry_ + [entry] for entry_ in self.entries]
            return

        entries = []
        if not self.entries and not subresult.entries:
            pass

        if not self.entries:
            entries = [entry * times for entry in subresult.entries]
        elif not subresult.entries:
            entries = self.entries.copy()
        else:
            for a_entry in self.entries:
                for b_entry in subresult.entries:
                    entries.append(a_entry + b_entry * times)

        self.entries = entries
        if subresult.rtype == KeyScanResult.BoundSize:
            self.rtype = KeyScanResult.BoundSize

    def increase_by_possibly_infinity(self):
        self.rtype = KeyScanResult.PossiblyInfinite
        self.entries.clear()

    def max(self, other):
        if self.rtype == KeyScanResult.PossiblyInfinite or other.rtype == KeyScanResult.PossiblyInfinite:
            return KeyScanner.infinity()

        return KeyScanner(entries=self.entries + other.entries, rtype=KeyScanResult.BoundSize)


# Magic numbers used in XCDR1 parameter list encoding
class XCDR1Constants:
    PL_SHORT_MAX_PARAM_ID = 0x3f00      # Maximum parameter ID that can be used with short PL encoding
    PL_SHORT_MAX_PARAM_LEN = 0xffff     # Maximum parameter length that can be used with short PL encoding
    PL_SHORT_PID_EXTENDED = 0x3f01      # Indicates the extended (long) PL encoding is used
    PL_SHORT_PID_LIST_END = 0x3f02      # Indicates the end of the parameter list data structure
    PL_SHORT_PID_EXT_LEN = 0x8          # Value of the param header length field in case of extended PL encoding
    PL_SHORT_FLAG_IMPL_EXT = 0x8000     # Flag for implementation specific interpretation of the parameter (not implemented)
    PL_SHORT_FLAG_MU = 0x4000           # Flag to indicate the parameter is must-understand in short PL header

    # Mask for the member ID in the short PL header; we don't use implementation-defined parameter ids (except
    # in discovery data, but that's handled elsewhere anyway) and including this bit in the mask means we
    # automatically treat them as unrecognised ids
    PL_SHORT_PID_MASK = 0x3fff | PL_SHORT_FLAG_IMPL_EXT

    PL_LONG_FLAG_IMPL_EXT = 0x80000000  # Flag used for RTPS discovery data types
    PL_LONG_FLAG_MU = 0x40000000        # Flag to indicate the parameter is must-understand in extended PL header

    # Mask for the member ID in the long PL header
    PL_LONG_MID_MASK = 0x0fffffff | PL_LONG_FLAG_IMPL_EXT


# Magic numbers from dds_public_impl.h
class DataRepresentationFlags:
    FLAG_XCDR1 = (0x1 << 0)
    FLAG_XCDR2 = (0x1 << 2)

# Magic numbers from dds_data_type_properties.h
class DataTypeProperties:
    DEFAULTS_TO_XCDR2   = (0x1 << 10)
    CONTAINS_KEY        = (0x1 << 12)
    IS_MEMCPY_SAFE      = (0x1 << 63)
    # Python binding internal ones:
    DISALLOWS_XCDR1     = (0x1 << 48)
    DISALLOWS_XCDR2     = (0x1 << 49)
    PYTHON_FLAGS_MASK   = (DISALLOWS_XCDR1 | DISALLOWS_XCDR2)



"""
 * Copyright(c) 2021 to 2022 ZettaScale Technology and others
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License v. 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0, or the Eclipse Distribution License
 * v. 1.0 which is available at
 * http://www.eclipse.org/org/documents/edl-v10.php.
 *
 * SPDX-License-Identifier: EPL-2.0 OR BSD-3-Clause
"""
import dataclasses as _dataclasses
from typing import Any, Tuple, Type, TypeVar, Optional, Dict, Callable, Sequence
from enum import Enum

from .types import ValidUnionHolder
from ._main import IdlMeta, IdlUnionMeta, IdlBitmaskMeta, IdlEnumMeta
from ._support import Buffer, Endianness, SerializeKind


_TIS = TypeVar('_TIS', bound='IdlStruct')
_TIU = TypeVar('_TIU', bound='IdlUnion')
_TIB = TypeVar('_TIB', bound='IdlBitmask')
_TIE = TypeVar('_TIE', bound='IdlEnum')


class IdlStruct(metaclass=IdlMeta):
    def serialize(self, buffer: Optional[Buffer] = None, endianness: Optional[Endianness] = None, use_version_2: Optional[bool] = None) -> bytes:
        return self.__idl__.serialize(self, buffer=buffer, endianness=endianness, use_version_2=use_version_2)

    def serialize_key(self, endianness: Optional[Endianness] = None, use_version_2: Optional[bool] = None) -> bytes:
        return self.__idl__.serialize(self, endianness=endianness, use_version_2=use_version_2, serialize_kind=SerializeKind.KeyDefinitionOrder)

    @classmethod
    def deserialize(cls: Type[_TIS], data: bytes, has_header: bool = True, use_version_2: Optional[bool] = None) -> _TIS:
        return cls.__idl__.deserialize(data, has_header=has_header, use_version_2=use_version_2)

    @classmethod
    def deserialize_key(cls: Type[_TIS], data: bytes, has_header: bool = True, use_version_2: Optional[bool] = None) -> _TIS:
        return cls.__idl__.deserialize_key(data, has_header=has_header, use_version_2=use_version_2)

def make_idl_struct(class_name: str, typename: str, fields: Dict[str, Any], *, dataclassify=True,
                    field_annotations: Optional[Dict[str, Dict[str, Any]]] = None,
                    bases: Tuple[Type[IdlStruct], ...] = ()) -> Type[IdlStruct]:
    bases = tuple(list(bases) + [IdlStruct])
    namespace = IdlMeta.__prepare__(class_name, bases, typename=typename)

    for fieldname, _type in fields.items():
        namespace['__annotations__'][fieldname] = _type

    if field_annotations:
        namespace['__idl_field_annotations__'] = field_annotations

    cls = IdlMeta(class_name, bases, namespace)
    if dataclassify:
        cls = _dataclasses.dataclass(cls)
    return cls


class IdlUnion(metaclass=IdlUnionMeta):
    def __init__(self, **kwargs):
        self.__active = None
        self.__discriminator = None
        self.__value = None

        if len(kwargs) == 2 and 'discriminator' in kwargs and 'value' in kwargs:
            self.set(kwargs['discriminator'], kwargs['value'])
        elif len(kwargs) == 1:
            for key, value in kwargs.items():
                self.__setattr__(key, value)
        else:
            raise ValueError("Can only set one field of union.")

    def __setattr__(self, name: str, value: Any) -> None:
        if name not in self.__idl_names__:
            return super().__setattr__(name, value)

        if self.__idl_default__ and self.__idl_default__[0] == name:
            if self.__active:
                super().__setattr__(self.__active, None)

            self.__active = name
            self.__discriminator = None
            self.__value = value
            return super().__setattr__(name, value)

        for label, case in self.__idl_cases__.items():
            if case[0] == name:
                if self.__active:
                    super().__setattr__(self.__active, None)

                self.__active = name
                self.__discriminator = label
                self.__value = value
                return super().__setattr__(name, value)

        raise Exception("Programmer error, should not get here.")

    def __getattr__(self, name: str) -> Any:
        if name in self.__idl_names__ and not self.__active == name:
            raise AttributeError("Tried to get inactive case on union")
        return super().__getattribute__(name)

    def set(self, discriminator: int, value: Any) -> None:
        if discriminator not in self.__idl_cases__:
            if self.__active:
                super().__setattr__(self.__active, None)

            self.__discriminator = discriminator
            self.__value = value

            if self.__idl_default__:
                self.__active = self.__idl_default__[0]
                return super().__setattr__(self.__idl_default__[0], value)
        else:
            case = self.__idl_cases__[discriminator]
            if self.__active:
                super().__setattr__(self.__active, None)

            self.__active = case[0]
            self.__discriminator = discriminator
            self.__value = value
            return super().__setattr__(case[0], value)

    def get(self):
        return self.__discriminator, self.__value

    @property
    def discriminator(self) -> Optional[int]:
        return self.__discriminator

    @property
    def value(self):
        return self.__value

    def __repr__(self):
        return f"{self.__class__.__name__}[Union]{self.get()}"

    def __rich_repr__(self):
        if self.discriminator is None:
            yield "Default selected", None
        elif self.value is None:
            yield "Default selected", self.discriminator
        else:
            yield "discriminator", self.discriminator
            yield self.__active or "value", self.value

    def __str__(self):
        return self.__repr__()

    def __hash__(self) -> int:
        return hash(self.get())

    def __eq__(self, other):
        if self.__class__ != other.__class__:
            return False
        if self.discriminator != other.discriminator:
            return False
        if self.value != other.value:
            return False
        return True

    def serialize(self, buffer: Optional[Buffer] = None, endianness: Optional[Endianness] = None, use_version_2: Optional[bool] = None) -> bytes:
        return self.__idl__.serialize(self, buffer=buffer, endianness=endianness, use_version_2=use_version_2)

    @classmethod
    def deserialize(cls: Type[_TIU], data: bytes, has_header: bool = True, use_version_2: Optional[bool] = None) -> _TIU:
        return cls.__idl__.deserialize(data, has_header=has_header, use_version_2=use_version_2)

    @classmethod
    def deserialize_key(cls: Type[_TIU], data: bytes, has_header: bool = True, use_version_2: Optional[bool] = None) -> _TIU:
        return cls.__idl__.deserialize_key(data, has_header=has_header, use_version_2=use_version_2)


def make_idl_union(class_name: str, typename: str, fields: Dict[str, ValidUnionHolder],
                   discriminator: type, *,
                   field_annotations: Optional[Dict[str, Dict[str, Any]]] = None,
                   discriminator_is_key: bool = False) -> Type[IdlUnion]:
    namespace = IdlUnionMeta.__prepare__(
        class_name, (IdlUnion,),
        typename=typename,
        discriminator=discriminator,
        discriminator_is_key=discriminator_is_key
    )

    for fieldname, _type in fields.items():
        namespace['__annotations__'][fieldname] = _type

    if field_annotations:
        namespace['__idl_field_annotations__'] = field_annotations

    return IdlUnionMeta(class_name, (IdlUnion,), namespace)


class IdlBitmask(metaclass=IdlBitmaskMeta):
    @classmethod
    def from_mask(cls: Type[_TIB], mask: int) -> _TIB:
        values = {}
        for fmask, name in cls.__idl_bits__.items():
            values[name] = (mask & fmask) > 0
        return cls(**values)

    def as_mask(self) -> int:
        return sum(mask for mask, name in self.__idl_bits__.items() if getattr(self, name))

    def __eq__(self, other):
        return self.__class__ == other.__class__ and self.as_mask() == other.as_mask()


def make_idl_bitmask(class_name: str, typename: str, fields: Sequence[str], *, dataclassify=True,
                     field_annotations: Optional[Dict[str, Dict[str, Any]]] = None) -> Type[IdlBitmask]:

    namespace = IdlBitmaskMeta.__prepare__(class_name, (IdlBitmask,), typename=typename)

    for fieldname in fields:
        namespace['__annotations__'][fieldname] = bool

    if field_annotations:
        namespace['__idl_field_annotations__'] = field_annotations

    cls = IdlBitmaskMeta(class_name, (IdlBitmask,), namespace)
    if dataclassify:
        cls = _dataclasses.dataclass(cls)
    return cls


class IdlEnum(Enum, metaclass=IdlEnumMeta):
    def _generate_next_value_(name, start, count, last_values):
        return last_values[-1] + 1 if last_values else count

    def __hash__(self) -> int:
        return hash(self.value)


def make_idl_enum(class_name: str, typename: str, fields: Dict[str, int],
                  default: Optional[str] = None) -> Type[IdlEnum]:
    namespace = IdlEnumMeta.__prepare__(class_name, (IdlEnum,), typename=typename)

    for fieldname, value in fields.items():
        namespace[fieldname] = value

    return IdlEnumMeta(class_name, (IdlEnum,), namespace)


__all__ = [
    "IdlUnion", "IdlStruct", "IdlBitmask", "IdlEnum",
    "make_idl_struct", "make_idl_union", "make_idl_bitmask",
    "make_idl_enum"
]



import pytest

from cyclonedds.domain import Domain, DomainParticipant


def test_domain_initialize():
    domain = Domain(1)

def test_domain_lookup():
    domain = Domain(1)
    dp = DomainParticipant(1)

    pc = domain.get_participants()
    assert len(pc) == 1 and pc[0] == dp


"""
 * Copyright(c) 2021 ZettaScale Technology and others
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License v. 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0, or the Eclipse Distribution License
 * v. 1.0 which is available at
 * http://www.eclipse.org/org/documents/edl-v10.php.
 *
 * SPDX-License-Identifier: EPL-2.0 OR BSD-3-Clause
"""

import typing as _typing
from . import _type_helper as _th


def _type_repr(obj):
    """Avoid printing <class 'int'>"""
    if type(obj) == str:
        return obj
    if isinstance(obj, type):
        if obj.__module__ == 'builtins':
            return obj.__qualname__
        return f'{obj.__module__}.{obj.__qualname__}'
    if obj is ...:
        return '...'
    if _th.get_origin(obj) == _th.Annotated:
        return _type_repr(_th.get_args(obj)[1])
    return repr(obj)


class array:
    @classmethod
    def __class_getitem__(cls, tup):
        if type(tup) != tuple:
            tup = (tup,)

        if len(tup) != 2 or type(tup[1]) != int:
            raise TypeError("An array takes two arguments: an element type and a constant length.")
        return _th.Annotated[_typing.Sequence[tup[0]], cls(*tup)]

    def __init__(self, subtype: type, length: int):
        self.subtype: type = subtype
        self.length: int = length

    def __repr__(self) -> str:
        return f"array[{_type_repr(self.subtype)}, {self.length}]"

    def __eq__(self, o: object) -> bool:
        return isinstance(o, array) and o.subtype == self.subtype and o.length == self.length

    def __hash__(self) -> int:
        return (241813571056069 * self.length) ^ hash(self.subtype)

    __str__ = __repr__


class sequence:
    @classmethod
    def __class_getitem__(cls, tup):
        if type(tup) != tuple:
            tup = (tup,)

        if len(tup) not in [1, 2] or (len(tup) == 2 and type(tup[1]) != int):
            raise TypeError("A sequence takes a subtype and an optional maximum length.")
        if len(tup) > 1 and (tup[1] <= 0 or tup[1] > 65535):
            return TypeError("Sequence max length should be between 0 and 65536.")
        return _th.Annotated[_typing.Sequence[tup[0]], cls(*tup)]

    def __init__(self, subtype: type, max_length: _typing.Optional[int] = None) -> None:
        self.subtype: type = subtype
        self.max_length: _typing.Optional[int] = max_length

    def __repr__(self) -> str:
        if self.max_length:
            return f"sequence[{_type_repr(self.subtype)}, {self.max_length}]"
        else:
            return f"sequence[{_type_repr(self.subtype)}]"

    def __eq__(self, o: object) -> bool:
        return isinstance(o, sequence) and o.subtype == self.subtype and o.max_length == self.max_length

    def __hash__(self) -> int:
        if self.max_length:
            return (406820761152607 * self.max_length) ^ hash(self.subtype)
        else:
            return 406820761152609 ^ hash(self.subtype)

    __str__ = __repr__


class typedef:
    @classmethod
    def __class_getitem__(cls, tup):
        if type(tup) != tuple:
            tup = (tup,)

        if len(tup) != 2:
            raise TypeError("A typedef takes a name and real type as argument.")

        rtype = tup[1]
        while _th.get_origin(rtype) == _th.Annotated:
            rtype = _th.get_args(rtype)[0]
        return _th.Annotated[rtype, cls(*tup)]

    def __init__(self, name: str, subtype: type) -> None:
        self.name: str = name
        self.subtype: type = subtype

    def __repr__(self) -> str:
        return f"typedef[{self.name}, {_type_repr(self.subtype)}]"

    def __eq__(self, o: object) -> bool:
        return (isinstance(o, typedef) and o.subtype == self.subtype)

    def __hash__(self) -> int:
        return hash(self.name) ^ hash(self.subtype)

    def __call__(self, *args: _typing.Any, **kwds: _typing.Any) -> _typing.Any:
        return self.subtype(*args, **kwds)

    @property
    def __idl_typename__(self):
        return self.name

    __str__ = __repr__


class bounded_str:
    @classmethod
    def __class_getitem__(cls, tup):
        if type(tup) != tuple:
            tup = (tup,)

        if len(tup) != 1 or type(tup[0]) != int:
            raise TypeError("A bounded str takes one argument, a maximum length.")
        if tup[0] <= 0:
            return TypeError("Bound string max length should be bigger than 0.")
        return _th.Annotated[str, cls(*tup)]

    def __init__(self, max_length: int) -> None:
        self.max_length: int = max_length

    def __repr__(self) -> str:
        return f"bounded_str[{self.max_length}]"

    def __eq__(self, o: object) -> bool:
        return isinstance(o, bounded_str) and o.max_length == self.max_length

    def __hash__(self) -> int:
        return 277146416319491 * self.max_length

    __str__ = __repr__


class ValidUnionHolder:
    pass


class case(ValidUnionHolder):
    @classmethod
    def __class_getitem__(cls, tup):
        if type(tup) != tuple:
            tup = (tup,)

        if len(tup) != 2:
            raise TypeError("A case takes two arguments: the discriminator value(s) and the type.")
        return _th.Annotated[_typing.Optional[tup[1]], cls(*tup)]

    def __init__(self, discriminator_value, subtype: type) -> None:
        self.labels: _typing.List[int] = discriminator_value if type(discriminator_value) == list else [discriminator_value]
        self.subtype: type = subtype

    def __repr__(self) -> str:
        return f"case[{self.labels}, {_type_repr(self.subtype)}]"

    def __eq__(self, o: object) -> bool:
        return isinstance(o, case) and o.subtype == self.subtype and o.labels == self.labels

    def __hash__(self) -> int:
        return hash(self.subtype)

    __str__ = __repr__


class default(ValidUnionHolder):
    @classmethod
    def __class_getitem__(cls, tup):
        if type(tup) != tuple:
            tup = (tup,)

        if len(tup) != 1:
            raise TypeError("A default takes one arguments: the type.")
        return _th.Annotated[_typing.Optional[tup[0]], cls(*tup)]

    def __init__(self, subtype: type) -> None:
        self.subtype: type = subtype

    def __repr__(self) -> str:
        return f"default[{_type_repr(self.subtype)}]"

    def __eq__(self, o: object) -> bool:
        return isinstance(o, default) and o.subtype == self.subtype

    def __hash__(self) -> int:
        return hash(self.subtype)

    __str__ = __repr__


char = _th.Annotated[str, "char"]
"""The C ``char`` datatype. In Python this is implemented as a single-character string."""

wchar = _th.Annotated[int, "wchar"]
"""The C ``wchar`` datatype. Do not use, here for completeness."""

int8 = _th.Annotated[int, "int8"]
"""A signed 8 bit integer."""

int16 = _th.Annotated[int, "int16"]
"""A signed 16 bit integer."""

int32 = _th.Annotated[int, "int32"]
"""A signed 32 bit integer."""

int64 = _th.Annotated[int, "int64"]
"""A signed 64 bit integer."""

byte = _th.Annotated[int, "byte"]
"""An unsigned 8 bit integer."""

uint8 = _th.Annotated[int, "uint8"]
"""An unsigned 8 bit integer."""

uint16 = _th.Annotated[int, "uint16"]
"""An unsigned 16 bit integer."""

uint32 = _th.Annotated[int, "uint32"]
"""An unsigned 32 bit integer."""

uint64 = _th.Annotated[int, "uint64"]
"""An unsigned 64 bit integer."""

float32 = _th.Annotated[float, "float32"]
"""A 32bit floating point number. In typical C this is a regular ``float``."""

float64 = _th.Annotated[float, "float64"]
"""A 64bit floating point number. In typical C this is a regular ``float``."""

NoneType = type(None)
"""The NoneType, or a "void" type. This is not included in the OMG IDL spec or in the C library but it can be very useful."""

_type_code_align_size_default_mapping = {
    int8: ('b', 1, 1, 0),
    int16: ('h', 2, 2, 0),
    int32: ('i', 4, 4, 0),
    int64: ('q', 8, 8, 0),
    uint8: ('B', 1, 1, 0),
    uint16: ('H', 2, 2, 0),
    uint32: ('I', 4, 4, 0),
    uint64: ('Q', 8, 8, 0),
    byte: ('B', 1, 1, 0),
    float32: ('f', 4, 4, 0.0),
    float64: ('d', 8, 8, 0.0),
    wchar: ('h', 2, 2, 0),
    int: ('q', 8, 8, 0),
    bool: ('?', 1, 1, False),
    float: ('d', 8, 8, 0.0),
}



#!/usr/bin/env python

"""
 * Copyright(c) 2022 ZettaScale Technology and others
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License v. 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0, or the Eclipse Distribution License
 * v. 1.0 which is available at
 * http://www.eclipse.org/org/documents/edl-v10.php.
 *
 * SPDX-License-Identifier: EPL-2.0 OR BSD-3-Clause
"""

import os
import sys
from pathlib import Path
from setuptools import setup, Extension, find_packages

this_directory = Path(__file__).resolve().parent
sys.path.insert(0, str(this_directory / 'buildhelp'))

from cyclone_search import find_cyclonedds
from build_ext import build_ext, Library
from bdist_wheel import bdist_wheel


__version__ = "11.0.1"
package = "cyclonedds"


with open(this_directory / 'README.md', encoding='utf-8') as f:
    long_description = f.read()


if "BUILDING_SDIST" not in os.environ:
    cyclone = find_cyclonedds()

    if not cyclone:
        print("Could not locate cyclonedds. Try to set CYCLONEDDS_HOME or CMAKE_PREFIX_PATH")
        import sys
        sys.exit(1)


    if "PYOXIDIZER" in os.environ:
        (this_directory / 'cyclonedds' / '__library__.py').write_text((this_directory / 'buildhelp' / 'oxidize_library.py').read_text())
    else:
        with open(this_directory / 'cyclonedds' / '__library__.py', "w", encoding='utf-8') as f:
            f.write("in_wheel = False\n")
            f.write(f"library_path = r'{cyclone.ddsc_library}'\n")

    ext_modules = [
        Extension('cyclonedds._clayer', [
                'clayer/pysertype.c'
            ],
            include_dirs=[
                str(cyclone.include_path),
                str(this_directory / "clayer")
            ],
            libraries=['ddsc'],
            library_dirs=[
                str(cyclone.library_path),
                str(cyclone.binary_path),
            ]
        )
    ]

    if cyclone.idlc_library:
        ext_modules += [
            Library('cyclonedds._idlpy', [
                    'idlpy/src/context.c',
                    'idlpy/src/generator.c',
                    'idlpy/src/naming.c',
                    'idlpy/src/ssos.c',
                    'idlpy/src/types.c',
                    'idlpy/src/util.c'
                ],
                include_dirs=[
                    str(cyclone.include_path),
                    str(this_directory / "idlpy" / "include")
                ],
                libraries=['ddsc', 'cycloneddsidl'],
                library_dirs=[
                    str(cyclone.library_path),
                    str(cyclone.binary_path)
                ]
            )
        ]
else:
    ext_modules=[]


setup(
    name=package,
    version=__version__,
    description='Eclipse Cyclone DDS Python binding',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='Eclipse Cyclone DDS Committers',
    maintainer='Thijs Miedema',
    maintainer_email='thijs.miedema@zettascale.tech',
    url="https://cyclonedds.io",
    project_urls={
        "Documentation": "https://cyclonedds.io/docs",
        "Source Code": "https://github.com/eclipse-cyclonedds/cyclonedds-python",
        "Bug Tracker": "https://github.com/eclipse-cyclonedds/cyclonedds-python/issues"
    },
    license="EPL-2.0, BSD-3-Clause",
    platforms=["Windows", "Linux", "Mac OS-X", "Unix"],
    keywords=[
        "eclipse", "cyclone", "dds", "pub", "sub",
        "pubsub", "iot", "cyclonedds", "cdr", "omg",
        "idl", "middleware", "ros"
    ],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        #"License :: OSI Approved :: Eclipse Public License 2.0 (EPL-2.0)",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Programming Language :: Python :: 3.13",
        "Operating System :: OS Independent"
    ],
    packages=find_packages(".", include=("cyclonedds", "cyclonedds.*")),
    package_data={
        "cyclonedds": ["*.so", "*.dylib", "*.dll", "idlc*", "*py.typed"],
        "cyclonedds.idl": ["py.typed"]
    },
    ext_modules=ext_modules,
    cmdclass={
        'bdist_wheel': bdist_wheel,
        'build_ext': build_ext,
    },
    entry_points={
        "console_scripts": [
            "ddsls=cyclonedds.tools.ddsls:command",
            "pubsub=cyclonedds.tools.pubsub:command",
            "cyclonedds=cyclonedds.tools.cli.main:cli"
        ],
    },
    python_requires='>=3.10',
    install_requires=[
        "rich-click"
    ],
    extras_require={
        "dev": [
            "pytest>=6.2",
            "pytest-asyncio",
            "pytest-cov",
            "pytest-mock",
            "flake8",
            "flake8-bugbear",
            "twine"
        ],
        "docs": [
            "Sphinx>=4.0.0",
            "piccolo_theme>=0.12.0"
        ]
    },
    zip_safe=False
)



import pytest

from cyclonedds.core import Entity, DDSStatus

from support_modules.testtopics import Message


def test_communication_basic_read(common_setup):
    msg = Message(message="Hi!")
    common_setup.dw.write(msg)
    result = common_setup.dr.read()

    assert len(result) == 1
    assert result[0] == msg


def test_communication_basic_take(common_setup):
    msg = Message(message="Hi!")
    common_setup.dw.write(msg)
    result = common_setup.dr.take()

    assert len(result) == 1
    assert result[0] == msg


def test_communication_order(common_setup):
    msg1 = Message(message="Hi1!")
    msg2 = Message(message="Hi2!")
    common_setup.dw.write(msg1)
    common_setup.dw.write(msg2)
    result = common_setup.dr.read(N=2)
    
    assert len(result) == 2
    assert result[0] == msg1
    assert result[1] == msg2


def test_communication_read_nodestroys(common_setup):
    msg = Message(message="Hi!")
    common_setup.dw.write(msg)
    common_setup.dr.read()
    result = common_setup.dr.read()

    assert len(result) == 1
    assert result[0] == msg


def test_communication_take_destroys(common_setup):
    msg = Message(message="Hi!")
    common_setup.dw.write(msg)
    result1 = common_setup.dr.read()
    result2 = common_setup.dr.take()
    result3 = common_setup.dr.read()

    assert len(result1) == 1
    assert len(result2) == 1
    assert len(result3) == 0
    assert result1[0] == result2[0] == msg


def test_communication_status_mask(common_setup):
    common_setup.dr.set_status_mask(DDSStatus.SubscriptionMatched)
    status = common_setup.dr.read_status()
    assert status == DDSStatus.SubscriptionMatched

    del common_setup.dw

    status = common_setup.dr.get_status_changes()
    assert (status & DDSStatus.SubscriptionMatched) > 0

    status = common_setup.dr.take_status()
    assert (status & DDSStatus.SubscriptionMatched) > 0



import rich_click as click

from .settings import CONTEXT_SETTINGS
from .ls import ls
from .ps import ps
from .typeof import typeof
from .sub import subscribe
from .pub import publish
from .ddsperf import performance


@click.group(context_settings=CONTEXT_SETTINGS)
def cli():
    # Initialize the CLI group
    pass


cli.add_command(ls)
cli.add_command(ps)
cli.add_command(typeof)
cli.add_command(subscribe)
cli.add_command(publish)
cli.add_command(performance)

if __name__ == "__main__":
    cli()
